# Copyright 2016 Cisco Systems.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


import collections

import mock

from neutron.tests import base

from networking_cisco.apps.saf.common import config
from networking_cisco.apps.saf.common import utils
from networking_cisco.apps.saf.db import dfa_db_models as dbm
from networking_cisco.apps.saf.server.services.firewall.native import (
    fabric_setup_base as fsb)
import networking_cisco.apps.saf.server.services.firewall.native.fw_constants \
    as fw_const

TENANT_NAME = 'TenantA'
TENANT_ID = '0000-1111-2222-5555'
FW_ID = '0000-aaaa-bbbb-ccce'
NET_ID = '0000-1111-bbbb-ccce'
OUT_NET_ID = '0000-1112-bbbb-ccce'
RTR_NET_ID = '0000-1113-bbbb-ccce'
SUBNET_ID = '0000-2222-bbbb-ccce'
OUT_SUBNET_ID = '0000-2222-bbbc-ccce'
RTR_SUBNET_ID = '0000-2222-bbbd-ccce'
FW_NAME = 'FwA'
POLCY_ID = '0000-aaaa-bbbb-cccc'
FW_TYPE = 'TE'
ROUTER_ID = '0000-aaaa-bbbb-5555'
IN_SUBNET = '100.100.11.0'
IN_SUBNET_AND_MASK = '100.100.11.0/24'
IN_START = '100.100.11.3'
IN_SEC_GW = '100.100.11.254'
IN_GW = '100.100.11.2'
IN_DCNM_GW = '100.100.11.1'
IN_END = '100.100.11.254'
SEGMENTATION_ID = 87999
OUT_SUBNET = '200.200.11.0'
OUT_SUBNET_AND_MASK = '200.200.11.0/24'
RTR_SUBNET_AND_MASK = '9.9.9.0/24'
OUT_START = '200.200.11.3'
OUT_SEC_GW = '200.200.11.254'
OUT_GW = '200.200.11.2'
OUT_DCNM_GW = '200.200.11.1'
OUT_END = '200.200.11.254'
OUT_SEGMENTATION_ID = 88000
EXT_PART = 34500

VLAN_ID = 0


try:
    OrderedDict = collections.OrderedDict
except AttributeError:
    import ordereddict
    OrderedDict = ordereddict.OrderedDict


class FakeClass(object):
    """Fake class. """
    @classmethod
    def imitate(cls, *others):
        for other in others:
            for name in other.__dict__:
                try:
                    setattr(cls, name, mock.Mock())
                except (TypeError, AttributeError):
                    pass
        return cls

    @classmethod
    def set_return(cls, class_name, fn_name, return_val):
        getattr(cls, fn_name).return_value = return_val


class FabricBaseTest(base.BaseTestCase):
    """A test suite to exercise the Fabric setup Base. """

    def setUp(self):
        '''Setup for the test scripts '''
        super(FabricBaseTest, self).setUp()
        self._init_values()
        config = self._fill_cfg()
        self.cfg = config
        self.cfg = config.CiscoDFAConfig().cfg

        fsb.FabricBase.__bases__ = (FakeClass.imitate(dbm.DfaDBMixin,
                                                      fsb.FabricApi),)
        FakeClass.set_return(dbm.DfaDBMixin, 'get_all_fw_db', dict())
        mock.patch('networking_cisco.apps.saf.db.dfa_db_models.'
                   'DfaSegmentTypeDriver').start()
        mock.patch('networking_cisco.apps.saf.db.dfa_db_models.'
                   'DfasubnetDriver').start()
        mock.patch('networking_cisco.apps.saf.server.'
                   'dfa_openstack_helper.DfaNeutronHelper').start()
        mock.patch('networking_cisco.apps.saf.db.dfa_db_models.'
                   'DfaDBMixin.update_fw_db').start()
        self.upd_fw_db_res_mock = mock.patch(
            'networking_cisco.apps.saf.db.dfa_db_models.DfaDBMixin.'
            'update_fw_db_result').start()
        self.app_state_final_res_mock = mock.patch(
            'networking_cisco.apps.saf.db.dfa_db_models.DfaDBMixin.'
            'append_state_final_result').start()
        self.fabric_base = fsb.FabricBase()
        self.add_nwk_db_mock = mock.patch.object(self.fabric_base,
                                                 'add_network_db').start()
        self.fabric_base.store_dcnm(mock.MagicMock())

    def _init_values(self):
        self.tenant_name = TENANT_NAME
        self.tenant_id = TENANT_ID
        self.net_id = NET_ID
        self.out_net_id = OUT_NET_ID
        self.rtr_net_id = RTR_NET_ID
        self.subnet_id = SUBNET_ID
        self.out_subnet_id = OUT_SUBNET_ID
        self.rtr_subnet_id = RTR_SUBNET_ID
        self.fw_id = FW_ID
        self.fw_name = FW_NAME
        self.policy_id = POLCY_ID
        self.fw_type = FW_TYPE
        self.router_id = ROUTER_ID
        self.in_subnet = IN_SUBNET
        self.in_subnet_and_mask = IN_SUBNET_AND_MASK
        self.in_gw = IN_GW
        self.in_dcnm_gw = IN_DCNM_GW
        self.in_sec_gw = IN_SEC_GW
        self.in_start = IN_START
        self.in_end = IN_END
        self.segmentation_id = SEGMENTATION_ID
        self.in_srvc_nwk_name = self.fw_id[0:4] + fw_const.IN_SERVICE_NWK + (
            self.fw_id[len(self.fw_id) - 4:])
        self.out_subnet = OUT_SUBNET
        self.out_subnet_and_mask = OUT_SUBNET_AND_MASK
        self.out_gw = OUT_GW
        self.out_dcnm_gw = OUT_DCNM_GW
        self.out_sec_gw = OUT_SEC_GW
        self.out_start = OUT_START
        self.out_end = OUT_END
        self.out_segmentation_id = OUT_SEGMENTATION_ID
        self.out_srvc_nwk_name = self.fw_id[0:4] + fw_const.OUT_SERVICE_NWK + (
            self.fw_id[len(self.fw_id) - 4:])
        self.mock_fw_dict = self._prepare_fw_dict()
        self.net_dict = self._prepare_net_dict("in")
        self.out_net_dict = self._prepare_net_dict("out")
        self.rtr_net_dict = self._prepare_rtr_net_dict()
        self.rtr_subnet_and_mask = RTR_SUBNET_AND_MASK
        self.ext_part = EXT_PART

    def _fill_cfg(self):
        config.default_firewall_opts['firewall'][
            'fw_auto_serv_nwk_create'] = True
        config.default_firewall_opts['firewall'][
            'fw_service_host_profile'] = fw_const.HOST_PROF
        config.default_firewall_opts['firewall'][
            'fw_service_host_fwd_mode'] = fw_const.HOST_FWD_MODE
        config.default_firewall_opts['firewall'][
            'fw_service_ext_profile'] = fw_const.EXT_PROF
        config.default_firewall_opts['firewall'][
            'fw_service_ext_fwd_mode'] = fw_const.EXT_FWD_MODE
        config.default_firewall_opts['firewall'][
            'fw_service_part_vrf_profile'] = fw_const.PART_PROF
        config.default_firewall_opts['firewall']['fw_mgmt_ip'] = '1.1.1.1'
        config.default_dcnm_opts['dcnm']['vlan_id_min'] = 2
        config.default_dcnm_opts['dcnm']['vlan_id_max'] = 200
        config.default_dcnm_opts['dcnm']['segmentation_id_min'] = 20000
        config.default_dcnm_opts['dcnm']['segmentation_id_max'] = 30000
        config.default_dcnm_opts['dcnm']['segmentation_reuse_timeout'] = 20
        return config

    def test_fabric_base_init(self):
        '''Wrapper for the init'''
        pass

    def _prepare_fw_dict(self):
        mock_fw_dict = {'rules': {}, 'tenant_name': self.tenant_name,
                        'tenant_id': self.tenant_id, 'fw_id': self.fw_id,
                        'fw_name': self.fw_name,
                        'firewall_policy_id': self.policy_id,
                        'fw_type': self.fw_type, 'router_id': self.router_id}
        return mock_fw_dict

    def _prepare_net_dict(self, direc):
        if direc == 'in':
            srvc_name = self.in_srvc_nwk_name
            srvc_seg = SEGMENTATION_ID
            srvc_prof = config.default_firewall_opts[
                'firewall']['fw_service_host_profile']
            srvc_fwd_mode = config.default_firewall_opts[
                'firewall']['fw_service_host_fwd_mode']
        else:
            srvc_name = self.out_srvc_nwk_name
            srvc_seg = OUT_SEGMENTATION_ID
            srvc_prof = config.default_firewall_opts[
                'firewall']['fw_service_ext_profile']
            srvc_fwd_mode = config.default_firewall_opts[
                'firewall']['fw_service_ext_fwd_mode']
        network_dict = {'tenant_id': self.tenant_id, 'name': srvc_name,
                        'segmentation_id': srvc_seg, 'vlan': VLAN_ID,
                        'config_profile': srvc_prof, 'fwd_mode': srvc_fwd_mode}
        return network_dict

    def _prepare_rtr_net_dict(self):
        rtr_nwk = self.fw_id[0:4] + fw_const.DUMMY_SERVICE_NWK + (
            self.fw_id[len(self.fw_id) - 4:])
        return {'tenant_id': self.tenant_id, 'name': rtr_nwk,
                'segmentation_id': None, 'vlan': None,
                'config_profile': None, 'fwd_mode': None}

    def _fill_fw_net_dict(self):
        return {'fw_id': self.fw_id, 'tenant_id': self.tenant_id,
                'name': self.fw_name, 'in_service_ip': self.in_start,
                'in_network_id': self.net_id}

    def _fill_fw_net_res_dict(self):
        fw_net_dict2 = self._fill_fw_net_dict()
        fw_net_dict2['os_status'] = fw_const.OS_IN_NETWORK_CREATE_SUCCESS
        return fw_net_dict2

    def _fill_fw_out_net_dict(self):
        return {'fw_id': self.fw_id, 'tenant_id': self.tenant_id,
                'name': self.fw_name, 'out_service_ip': self.out_start,
                'out_network_id': self.out_net_id}

    def _fill_fw_rtr_net_dict(self):
        return {'router_id': self.router_id, 'name': self.fw_name,
                'router_net_id': self.rtr_net_id,
                'tenant_id': self.tenant_id,
                'fw_id': self.fw_id,
                'router_subnet_id': self.rtr_subnet_id,
                'os_status': fw_const.OS_DUMMY_RTR_CREATE_SUCCESS}

    def _fill_fw_rtr_net_dict_virt(self):
        return {'router_id': self.router_id, 'name': self.fw_name,
                'router_net_id': None,
                'tenant_id': self.tenant_id,
                'fw_id': self.fw_id, 'router_subnet_id': None,
                'os_status': fw_const.OS_DUMMY_RTR_CREATE_SUCCESS}

    def _fill_fw_out_net_res_dict(self):
        fw_net_dict2 = self._fill_fw_out_net_dict()
        fw_net_dict2['os_status'] = fw_const.OS_OUT_NETWORK_CREATE_SUCCESS
        return fw_net_dict2

    def _fill_fw_dcnm_rest_net_dict(self):
        return {
            'status': 'ACTIVE', 'admin_state_up': True,
            'tenant_id': self.tenant_id, 'provider:network_type': 'local',
            'vlan_id': VLAN_ID, 'segmentation_id': self.segmentation_id,
            'mob_domain': False, 'mob_domain_name': None,
            'name': self.in_srvc_nwk_name, 'part_name': None,
            'config_profile': config.default_firewall_opts[
                'firewall']['fw_service_host_profile'],
            'fwd_mode': config.default_firewall_opts[
                'firewall']['fw_service_host_fwd_mode']}

    def _fill_fw_dcnm_rest_subnet_dict(self):
        name = self.fw_id[0:4] + fw_const.IN_SERVICE_SUBNET + (
            self.fw_id[len(self.fw_id) - 4:])
        alloc_pool_dict = {}
        alloc_pool_dict['start'] = self.in_start
        alloc_pool_dict['end'] = self.in_end
        fw_subnet_dict = {'name': name,
                          'enable_dhcp': False,
                          'tenant_id': self.tenant_id,
                          'cidr': self.in_subnet_and_mask,
                          'gateway_ip': self.in_dcnm_gw,
                          'secondary_gw': self.in_sec_gw,
                          'ip_version': 4}
        fw_subnet_dict['allocation_pools'] = []
        fw_subnet_dict['allocation_pools'].append(alloc_pool_dict)
        return fw_subnet_dict

    def _fill_fw_dcnm_rest_out_net_dict(self):
        return {
            'status': 'ACTIVE', 'admin_state_up': True,
            'tenant_id': self.tenant_id, 'provider:network_type': 'local',
            'vlan_id': VLAN_ID, 'segmentation_id': self.out_segmentation_id,
            'mob_domain': False, 'mob_domain_name': None,
            'name': self.out_srvc_nwk_name,
            'part_name': fw_const.SERV_PART_NAME,
            'config_profile': config.default_firewall_opts[
                'firewall']['fw_service_ext_profile'],
            'fwd_mode': config.default_firewall_opts[
                'firewall']['fw_service_ext_fwd_mode']}

    def _fill_fw_dcnm_rest_out_subnet_dict(self):
        name = self.fw_id[0:4] + fw_const.OUT_SERVICE_SUBNET + (
            self.fw_id[len(self.fw_id) - 4:])
        alloc_pool_dict = {}
        alloc_pool_dict['start'] = self.out_start
        alloc_pool_dict['end'] = self.out_end
        fw_subnet_dict = {'name': name,
                          'enable_dhcp': False,
                          'tenant_id': self.tenant_id,
                          'cidr': self.out_subnet_and_mask,
                          'gateway_ip': self.out_dcnm_gw,
                          'secondary_gw': self.out_sec_gw,
                          'ip_version': 4}
        fw_subnet_dict['allocation_pools'] = []
        fw_subnet_dict['allocation_pools'].append(alloc_pool_dict)
        return fw_subnet_dict

    def _fill_fw_dcnm_net_dict(self):
        return {'router_id': self.router_id,
                'out_network_id': self.out_net_id,
                'name': self.fw_name,
                'router_net_id': self.rtr_net_id,
                'tenant_id': self.tenant_id,
                'fw_id': self.fw_id,
                'dcnm_status': fw_const.DCNM_IN_NETWORK_CREATE_SUCCESS,
                'in_network_id': self.net_id,
                'out_service_ip': None,
                'os_status': fw_const.OS_DUMMY_RTR_CREATE_SUCCESS,
                'router_subnet_id': self.rtr_subnet_id,
                'in_service_ip': None}

    def _fill_fw_dcnm_out_net_dict(self):
        return {'router_id': self.router_id,
                'out_network_id': self.out_net_id,
                'name': self.fw_name,
                'router_net_id': self.rtr_net_id,
                'tenant_id': self.tenant_id,
                'fw_id': self.fw_id,
                'dcnm_status': fw_const.DCNM_OUT_NETWORK_CREATE_SUCCESS,
                'in_network_id': self.net_id,
                'out_network_id': self.out_net_id,
                'out_service_ip': None,
                'os_status': fw_const.OS_DUMMY_RTR_CREATE_SUCCESS,
                'router_subnet_id': self.rtr_subnet_id,
                'in_service_ip': None,
                'out_service_ip': None}

    def _fill_fw_dcnm_part_upd_dict(self, direc):
        dcnm_status = fw_const.DCNM_IN_PART_UPDATE_SUCCESS \
            if direc == "in" else fw_const.DCNM_OUT_PART_UPDATE_SUCCESS
        fw_part_dict = {'tenant_id': self.tenant_id,
                        'fw_id': self.fw_id,
                        'dcnm_status': dcnm_status,
                        'name': self.fw_name}
        return fw_part_dict

    def _fill_fw_dcnm_part_create_dict(self):
        return {'tenant_id': self.tenant_id,
                'fw_id': self.fw_id,
                'dcnm_status': fw_const.DCNM_OUT_PART_CREATE_SUCCESS,
                'name': self.fw_name}

    def _fill_fw_del_net_dict(self):
        return {'router_id': self.router_id, 'name': self.fw_name,
                'router_net_id': self.rtr_net_id,
                'tenant_id': self.tenant_id,
                'fw_id': self.fw_id, 'out_network_id': self.out_net_id,
                'in_network_id': self.net_id, 'in_service_ip': None,
                'out_service_ip': None,
                'router_subnet_id': self.rtr_subnet_id}

    def _fill_fw_db_data(self, state):
        fw_data = dict()
        compl_result = fw_const.RESULT_FW_CREATE_INIT + '(' + str(state) + ')'
        fw_data[self.fw_id] = {
            'tenant_id': self.tenant_id, 'name': self.fw_name,
            'in_network_id': self.net_id, 'out_network_id': self.out_net_id,
            'os_status': fw_const.OS_DUMMY_RTR_CREATE_SUCCESS,
            'result': compl_result, 'router_id': self.router_id,
            'router_net_id': self.rtr_net_id,
            'router_subnet_id': self.rtr_subnet_id}
        return fw_data

    def test_create_in_nwk(self):
        """Create IN Network. """
        id_list = []
        id_list.append(self.net_id)
        id_list.append(self.subnet_id)
        fw_net_dict2 = self._fill_fw_net_res_dict()
        with mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                        'native.fabric_setup_base.ServiceIpSegTenantMap.'
                        'get_state',
                        return_value=fw_const.OS_IN_NETWORK_STATE),\
            mock.patch.object(self.fabric_base.os_helper, 'create_network',
                              return_value=id_list) as create_nwk:
            FakeClass.set_return(fsb.FabricApi, 'get_in_ip_addr',
                                 {'subnet': self.in_subnet,
                                  'start': self.in_start,
                                  'sec_gateway': self.in_sec_gw,
                                  'gateway': self.in_gw,
                                  'end': self.in_end})
            self.fabric_base.service_segs.allocate_segmentation_id.\
                return_value = self.segmentation_id
            self.fabric_base.fabric_fsm[fw_const.OS_OUT_NETWORK_STATE][0] = \
                mock.MagicMock()
            self.fabric_base.fabric_fsm[fw_const.OS_OUT_NETWORK_STATE][0].\
                return_value = False
            # update_fw_db is removed from here. Because both this and
            # update_fw_db_result uses the same fw_dict and fw_dict
            # is updated with result before calling update_fw_db_result. But,
            # mock records the updated fw_dict as a result of which argument
            # check fails.
            parent = mock.MagicMock()
            parent.attach_mock(create_nwk, 'create_network')
            parent.attach_mock(self.add_nwk_db_mock, 'add_network_db')
            parent.attach_mock(self.upd_fw_db_res_mock, 'update_fw_db_result')
            parent.attach_mock(self.app_state_final_res_mock,
                               'append_state_final_result')
            self.fabric_base.prepare_fabric_fw(self.tenant_id,
                                               self.mock_fw_dict, True,
                                               fw_const.RESULT_FW_CREATE_INIT)
        expected_calls = [mock.call.create_network(self.in_srvc_nwk_name,
                                                   self.tenant_id,
                                                   self.in_subnet_and_mask,
                                                   gw=self.in_gw),
                          mock.call.add_network_db(self.net_id, self.net_dict,
                                                   fw_const.FW_CONST,
                                                   'SUCCESS'),
                          mock.call.update_fw_db_result(
                              self.mock_fw_dict.get('fw_id'), fw_net_dict2),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_CREATE_INIT,
                              fw_const.OS_OUT_NETWORK_STATE),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_CREATE_INIT,
                              fw_const.OS_OUT_NETWORK_STATE)]
        parent.assert_has_calls(expected_calls, any_order=False)

    def test_create_in_nwk_fail(self):
        """Create IN Network Fail.

        The Openstack create network helper function is mocked to return a
        failure.
        """
        id_list = []
        with mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                        'native.fabric_setup_base.ServiceIpSegTenantMap.'
                        'get_state',
                        return_value=fw_const.OS_IN_NETWORK_STATE),\
            mock.patch.object(self.fabric_base.os_helper, 'create_network',
                              return_value=id_list) as create_nwk:
            FakeClass.set_return(fsb.FabricApi, 'get_in_ip_addr',
                                 {'subnet': self.in_subnet,
                                  'start': self.in_start,
                                  'sec_gateway': self.in_sec_gw,
                                  'gateway': self.in_gw,
                                  'end': self.in_end})
            self.fabric_base.service_segs.allocate_segmentation_id.\
                return_value = self.segmentation_id
            self.fabric_base.fabric_fsm[fw_const.OS_OUT_NETWORK_STATE][0] = \
                mock.MagicMock()
            self.fabric_base.fabric_fsm[fw_const.OS_OUT_NETWORK_STATE][0].\
                return_value = False
            # update_fw_db is removed from here. Because both this and
            # update_fw_db_result uses the same fw_dict and fw_dict
            # is updated with result before calling update_fw_db_result. But,
            # mock records the updated fw_dict as a result of which argument
            # check fails.
            parent = mock.MagicMock()
            parent.attach_mock(create_nwk, 'create_network')
            parent.attach_mock(self.add_nwk_db_mock, 'add_network_db')
            parent.attach_mock(self.upd_fw_db_res_mock, 'update_fw_db_result')
            parent.attach_mock(self.app_state_final_res_mock,
                               'append_state_final_result')
            self.fabric_base.prepare_fabric_fw(self.tenant_id,
                                               self.mock_fw_dict, True,
                                               fw_const.RESULT_FW_CREATE_INIT)
        expected_calls = [mock.call.create_network(self.in_srvc_nwk_name,
                                                   self.tenant_id,
                                                   self.in_subnet_and_mask,
                                                   gw=self.in_gw),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_CREATE_INIT,
                              fw_const.OS_IN_NETWORK_STATE)]
        parent.assert_has_calls(expected_calls, any_order=False)
        self.add_nwk_db_mock.assert_not_called()
        self.upd_fw_db_res_mock.assert_not_called()

    def test_create_out_nwk(self):
        """Create Out Network Test. """
        id_list = []
        id_list.append(self.out_net_id)
        id_list.append(self.out_subnet_id)
        fw_net_dict2 = self._fill_fw_out_net_res_dict()
        with mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                        'native.fabric_setup_base.ServiceIpSegTenantMap.'
                        'get_state',
                        return_value=fw_const.OS_OUT_NETWORK_STATE),\
            mock.patch.object(self.fabric_base.os_helper, 'create_network',
                              return_value=id_list) as create_nwk:
            FakeClass.set_return(fsb.FabricApi, 'get_out_ip_addr',
                                 {'subnet': self.out_subnet,
                                  'start': self.out_start,
                                  'sec_gateway': self.out_sec_gw,
                                  'gateway': self.out_gw,
                                  'end': self.out_end})
            self.fabric_base.service_segs.allocate_segmentation_id.\
                return_value = self.out_segmentation_id
            self.fabric_base.fabric_fsm[fw_const.OS_DUMMY_RTR_STATE][0] = \
                mock.MagicMock()
            self.fabric_base.fabric_fsm[fw_const.OS_DUMMY_RTR_STATE][0].\
                return_value = False
            # update_fw_db is removed from here. Because both this and
            # update_fw_db_result uses the same fw_dict and fw_dict
            # is updated with result before calling update_fw_db_result. But,
            # mock records the updated fw_dict as a result of which argument
            # check fails.
            parent = mock.MagicMock()
            parent.attach_mock(create_nwk, 'create_network')
            parent.attach_mock(self.add_nwk_db_mock, 'add_network_db')
            parent.attach_mock(self.upd_fw_db_res_mock, 'update_fw_db_result')
            parent.attach_mock(self.app_state_final_res_mock,
                               'append_state_final_result')
            self.fabric_base.prepare_fabric_fw(self.tenant_id,
                                               self.mock_fw_dict, True,
                                               fw_const.RESULT_FW_CREATE_INIT)
        expected_calls = [mock.call.create_network(self.out_srvc_nwk_name,
                                                   self.tenant_id,
                                                   self.out_subnet_and_mask,
                                                   gw=self.out_gw),
                          mock.call.add_network_db(self.out_net_id,
                                                   self.out_net_dict,
                                                   fw_const.FW_CONST,
                                                   'SUCCESS'),
                          mock.call.update_fw_db_result(
                              self.mock_fw_dict.get('fw_id'), fw_net_dict2),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_CREATE_INIT,
                              fw_const.OS_DUMMY_RTR_STATE),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_CREATE_INIT,
                              fw_const.OS_DUMMY_RTR_STATE)]
        parent.assert_has_calls(expected_calls, any_order=False)

    def test_create_out_nwk_fail(self):
        """Create OUT Network Fail.

        The Openstack create network helper function is mocked to return a
        failure.
        """
        id_list = [None, None]
        with mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                        'native.fabric_setup_base.ServiceIpSegTenantMap.'
                        'get_state',
                        return_value=fw_const.OS_OUT_NETWORK_STATE),\
            mock.patch.object(self.fabric_base.os_helper, 'create_network',
                              return_value=id_list) as create_nwk:
            FakeClass.set_return(fsb.FabricApi, 'get_out_ip_addr',
                                 {'subnet': self.out_subnet,
                                  'start': self.out_start,
                                  'sec_gateway': self.out_sec_gw,
                                  'gateway': self.out_gw,
                                  'end': self.out_end})
            self.fabric_base.service_segs.allocate_segmentation_id.\
                return_value = self.out_segmentation_id
            self.fabric_base.fabric_fsm[fw_const.OS_DUMMY_RTR_STATE][0] = \
                mock.MagicMock()
            self.fabric_base.fabric_fsm[fw_const.OS_DUMMY_RTR_STATE][0].\
                return_value = False
            # update_fw_db is removed from here. Because both this and
            # update_fw_db_result uses the same fw_dict and fw_dict
            # is updated with result before calling update_fw_db_result. But,
            # mock records the updated fw_dict as a result of which argument
            # check fails.
            parent = mock.MagicMock()
            parent.attach_mock(create_nwk, 'create_network')
            parent.attach_mock(self.add_nwk_db_mock, 'add_network_db')
            parent.attach_mock(self.upd_fw_db_res_mock, 'update_fw_db_result')
            parent.attach_mock(self.app_state_final_res_mock,
                               'append_state_final_result')
            self.fabric_base.prepare_fabric_fw(self.tenant_id,
                                               self.mock_fw_dict, True,
                                               fw_const.RESULT_FW_CREATE_INIT)
        expected_calls = [mock.call.create_network(self.out_srvc_nwk_name,
                                                   self.tenant_id,
                                                   self.out_subnet_and_mask,
                                                   gw=self.out_gw),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_CREATE_INIT,
                              fw_const.OS_OUT_NETWORK_STATE)]
        parent.assert_has_calls(expected_calls, any_order=False)
        self.add_nwk_db_mock.assert_not_called()
        self.upd_fw_db_res_mock.assert_not_called()

    def test_create_os_dummy_rtr(self):
        """Create Dummy Router Test. """
        id_list = []
        id_list.append(self.rtr_net_id)
        id_list.append(self.rtr_subnet_id)
        rtr_net_list = set()
        rtr_net_list.add(self.rtr_subnet_id)
        rtr_net_dict = self._fill_fw_rtr_net_dict()
        with mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                        'native.fabric_setup_base.ServiceIpSegTenantMap.'
                        'get_state',
                        return_value=fw_const.OS_DUMMY_RTR_STATE),\
            mock.patch.object(self.fabric_base.os_helper, 'create_network',
                              return_value=id_list) as create_nwk,\
            mock.patch.object(self.fabric_base.os_helper, 'add_intf_router',
                              return_value=True) as add_intf_rtr:
            self.fabric_base.fabric_fsm[fw_const.DCNM_IN_NETWORK_STATE][0] = \
                mock.MagicMock()
            self.fabric_base.fabric_fsm[fw_const.DCNM_IN_NETWORK_STATE][0].\
                return_value = False
            # update_fw_db is removed from here. Because both this and
            # update_fw_db_result uses the same fw_dict and fw_dict
            # is updated with result before calling update_fw_db_result. But,
            # mock records the updated fw_dict as a result of which argument
            # check fails.
            parent = mock.MagicMock()
            parent.attach_mock(create_nwk, 'create_network')
            parent.attach_mock(add_intf_rtr, 'add_intf_router')
            parent.attach_mock(self.add_nwk_db_mock, 'add_network_db')
            parent.attach_mock(self.upd_fw_db_res_mock, 'update_fw_db_result')
            parent.attach_mock(self.app_state_final_res_mock,
                               'append_state_final_result')
            self.fabric_base.prepare_fabric_fw(self.tenant_id,
                                               self.mock_fw_dict, False,
                                               fw_const.RESULT_FW_CREATE_INIT)
        rtr_nwk_name = self.fw_id[0:4] + fw_const.DUMMY_SERVICE_NWK + (
            self.fw_id[len(self.fw_id) - 4:])
        expected_calls = [mock.call.create_network(rtr_nwk_name,
                                                   self.tenant_id,
                                                   self.rtr_subnet_and_mask),
                          mock.call.add_network_db(self.rtr_net_id,
                                                   self.rtr_net_dict,
                                                   fw_const.FW_CONST,
                                                   'SUCCESS'),
                          mock.call.add_intf_router(self.router_id,
                                                    self.tenant_id,
                                                    rtr_net_list),
                          mock.call.update_fw_db_result(
                              self.mock_fw_dict.get('fw_id'), rtr_net_dict),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_CREATE_INIT,
                              fw_const.DCNM_IN_NETWORK_STATE),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_CREATE_INIT,
                              fw_const.DCNM_IN_NETWORK_STATE)]
        parent.assert_has_calls(expected_calls, any_order=False)

    def test_create_os_dummy_rtr_fail(self):
        """Create Dummy Router Fail Test.

        The Openstack add interface to router helper function is mocked to
        return a failure.
        """
        id_list = []
        id_list.append(self.rtr_net_id)
        id_list.append(self.rtr_subnet_id)
        rtr_net_list = set()
        rtr_net_list.add(self.rtr_subnet_id)
        with mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                        'native.fabric_setup_base.ServiceIpSegTenantMap.'
                        'get_state',
                        return_value=fw_const.OS_DUMMY_RTR_STATE),\
            mock.patch.object(self.fabric_base.os_helper, 'create_network',
                              return_value=id_list) as create_nwk,\
            mock.patch.object(self.fabric_base.os_helper, 'add_intf_router',
                              return_value=False) as add_intf_rtr,\
            mock.patch.object(self.fabric_base.os_helper, 'delete_network',
                              return_value=id_list):
            # update_fw_db is removed from here. Because both this and
            # update_fw_db_result uses the same fw_dict and fw_dict
            # is updated with result before calling update_fw_db_result. But,
            # mock records the updated fw_dict as a result of which argument
            # check fails.
            parent = mock.MagicMock()
            parent.attach_mock(create_nwk, 'create_network')
            parent.attach_mock(add_intf_rtr, 'add_intf_router')
            parent.attach_mock(self.add_nwk_db_mock, 'add_network_db')
            parent.attach_mock(self.upd_fw_db_res_mock, 'update_fw_db_result')
            parent.attach_mock(self.app_state_final_res_mock,
                               'append_state_final_result')
            self.fabric_base.prepare_fabric_fw(self.tenant_id,
                                               self.mock_fw_dict, False,
                                               fw_const.RESULT_FW_CREATE_INIT)
        rtr_nwk_name = self.fw_id[0:4] + fw_const.DUMMY_SERVICE_NWK + (
            self.fw_id[len(self.fw_id) - 4:])
        expected_calls = [mock.call.create_network(rtr_nwk_name,
                                                   self.tenant_id,
                                                   self.rtr_subnet_and_mask),
                          mock.call.add_network_db(self.rtr_net_id,
                                                   self.rtr_net_dict,
                                                   fw_const.FW_CONST,
                                                   'SUCCESS'),
                          mock.call.add_intf_router(self.router_id,
                                                    self.tenant_id,
                                                    rtr_net_list),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_CREATE_INIT,
                              fw_const.OS_DUMMY_RTR_STATE)]
        parent.assert_has_calls(expected_calls, any_order=False)
        self.upd_fw_db_res_mock.assert_not_called()

    def test_create_os_dummy_rtr_virt(self):
        """Create Dummy Router Virt Test. """
        id_list = []
        id_list.append(self.rtr_net_id)
        id_list.append(self.rtr_subnet_id)
        rtr_net_list = set()
        rtr_net_list.add(self.rtr_subnet_id)
        rtr_net_dict = self._fill_fw_rtr_net_dict_virt()
        with mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                        'native.fabric_setup_base.ServiceIpSegTenantMap.'
                        'get_state',
                        return_value=fw_const.OS_DUMMY_RTR_STATE),\
            mock.patch.object(self.fabric_base.os_helper, 'create_network',
                              return_value=id_list) as create_nwk,\
            mock.patch.object(self.fabric_base.os_helper, 'add_intf_router',
                              return_value=True) as add_intf_rtr:
            self.fabric_base.fabric_fsm[fw_const.DCNM_IN_NETWORK_STATE][0] = \
                mock.MagicMock()
            self.fabric_base.fabric_fsm[fw_const.DCNM_IN_NETWORK_STATE][0].\
                return_value = False
            # update_fw_db is removed from here. Because both this and
            # update_fw_db_result uses the same fw_dict and fw_dict
            # is updated with result before calling update_fw_db_result. But,
            # mock records the updated fw_dict as a result of which argument
            # check fails.
            parent = mock.MagicMock()
            parent.attach_mock(create_nwk, 'create_network')
            parent.attach_mock(add_intf_rtr, 'add_intf_router')
            parent.attach_mock(self.add_nwk_db_mock, 'add_network_db')
            parent.attach_mock(self.upd_fw_db_res_mock, 'update_fw_db_result')
            parent.attach_mock(self.app_state_final_res_mock,
                               'append_state_final_result')
            self.fabric_base.prepare_fabric_fw(self.tenant_id,
                                               self.mock_fw_dict, True,
                                               fw_const.RESULT_FW_CREATE_INIT)
        expected_calls = [
            mock.call.update_fw_db_result(self.mock_fw_dict.get('fw_id'),
                                          rtr_net_dict),
            mock.call.append_state_final_result(
                self.mock_fw_dict.get('fw_id'), fw_const.RESULT_FW_CREATE_INIT,
                fw_const.DCNM_IN_NETWORK_STATE),
            mock.call.append_state_final_result(
                self.mock_fw_dict.get('fw_id'), fw_const.RESULT_FW_CREATE_INIT,
                fw_const.DCNM_IN_NETWORK_STATE)]
        parent.assert_has_calls(expected_calls, any_order=False)
        create_nwk.assert_not_called()
        self.add_nwk_db_mock.assert_not_called()
        add_intf_rtr.assert_not_called()

    def test_create_dcnm_in_nwk(self):
        """Create IN Network Test.

        This function relies on the state information filled by previous
        functions. So, rather than starting fresh, we shall populate the
        FW DB. This is equivalent to restarting the enabler server and it
        reading the DB and populating the local cache
        """
        fw_dcnm_net_dict = self._fill_fw_dcnm_net_dict()
        fw_dcnm_rest_net_dict = self._fill_fw_dcnm_rest_net_dict()
        fw_dcnm_rest_subnet_dict = self._fill_fw_dcnm_rest_subnet_dict()
        fake_db_net = {'segmentation_id': self.segmentation_id,
                       'vlan': VLAN_ID}
        fake_net_obj = utils.Dict2Obj(fake_db_net)
        with mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                        'native.fabric_setup_base.ServiceIpSegTenantMap.'
                        'get_state',
                        return_value=fw_const.DCNM_IN_NETWORK_STATE),\
            mock.patch('networking_cisco.apps.saf.common.utils.'
                       'Dict2Obj') as dict_obj,\
            mock.patch.object(self.fabric_base.dcnm_obj,
                              'create_service_network') as create_nwk:
            self.fabric_base.fabric_fsm[
                fw_const.DCNM_IN_PART_UPDATE_STATE][0] = mock.MagicMock()
            self.fabric_base.fabric_fsm[
                fw_const.DCNM_IN_PART_UPDATE_STATE][0].return_value = False
            # update_fw_db is removed from here. Because both this and
            # update_fw_db_result uses the same fw_dict and fw_dict
            # is updated with result before calling update_fw_db_result. But,
            # mock records the updated fw_dict as a result of which argument
            # check fails.
            FakeClass.set_return(
                dbm.DfaDBMixin, 'get_all_fw_db',
                self._fill_fw_db_data(fw_const.DCNM_IN_NETWORK_STATE))
            FakeClass.set_return(dbm.DfaDBMixin, 'get_network', fake_net_obj)
            self.fabric_base.service_in_ip.get_subnet_by_netid.\
                return_value = self.in_subnet
            self.fabric_base.populate_local_cache()
            parent = mock.MagicMock()
            parent.attach_mock(dict_obj, 'Dict2Obj')
            parent.attach_mock(dict_obj, 'Dict2Obj')
            parent.attach_mock(self.upd_fw_db_res_mock, 'update_fw_db_result')
            parent.attach_mock(self.app_state_final_res_mock,
                               'append_state_final_result')
            self.fabric_base.prepare_fabric_fw(self.tenant_id,
                                               self.mock_fw_dict, True,
                                               fw_const.RESULT_FW_CREATE_INIT)
        expected_calls = [mock.call.Dict2Obj(fw_dcnm_rest_net_dict),
                          mock.call.Dict2Obj(fw_dcnm_rest_subnet_dict),
                          mock.call.update_fw_db_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_dcnm_net_dict),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_CREATE_INIT,
                              fw_const.DCNM_IN_PART_UPDATE_STATE),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_CREATE_INIT,
                              fw_const.DCNM_IN_PART_UPDATE_STATE)]
        parent.assert_has_calls(expected_calls, any_order=False)
        self.assertEqual(create_nwk.called, True)

    def test_create_dcnm_in_part_update(self):
        """DCNM Update IN Partition Test. """
        fw_dcnm_part_dict = self._fill_fw_dcnm_part_upd_dict("in")
        with mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                        'native.fabric_setup_base.ServiceIpSegTenantMap.'
                        'get_state',
                        return_value=fw_const.DCNM_IN_PART_UPDATE_STATE),\
            mock.patch.object(self.fabric_base.dcnm_obj,
                              'update_project') as update_proj:
            self.fabric_base.fabric_fsm[fw_const.DCNM_OUT_PART_STATE][0] = \
                mock.MagicMock()
            self.fabric_base.fabric_fsm[fw_const.DCNM_OUT_PART_STATE][0].\
                return_value = False
            # update_fw_db is removed from here. Because both this and
            # update_fw_db_result uses the same fw_dict and fw_dict
            # is updated with result before calling update_fw_db_result. But,
            # mock records the updated fw_dict as a result of which argument
            # check fails.
            FakeClass.set_return(fsb.FabricApi, 'get_in_ip_addr',
                                 {'subnet': self.in_subnet,
                                  'start': self.in_start,
                                  'sec_gateway': self.in_sec_gw,
                                  'gateway': self.in_gw,
                                  'end': self.in_end})
            parent = mock.MagicMock()
            parent.attach_mock(update_proj, 'update_project')
            parent.attach_mock(self.upd_fw_db_res_mock, 'update_fw_db_result')
            parent.attach_mock(self.app_state_final_res_mock,
                               'append_state_final_result')
            self.fabric_base.prepare_fabric_fw(self.tenant_id,
                                               self.mock_fw_dict, True,
                                               fw_const.RESULT_FW_CREATE_INIT)
        expected_calls = [mock.call.update_project(self.tenant_name, None,
                                                   desc='Service Partition',
                                                   service_node_ip=self.in_gw,
                                                   vrf_prof=None),
                          mock.call.update_fw_db_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_dcnm_part_dict),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_CREATE_INIT,
                              fw_const.DCNM_OUT_PART_STATE),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_CREATE_INIT,
                              fw_const.DCNM_OUT_PART_STATE)]
        parent.assert_has_calls(expected_calls, any_order=False)

    def test_create_dcnm_out_part(self):
        """DCNM Out Part Create Test. """
        fw_dcnm_part_dict = self._fill_fw_dcnm_part_create_dict()
        with mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                        'native.fabric_setup_base.ServiceIpSegTenantMap.'
                        'get_state',
                        return_value=fw_const.DCNM_OUT_PART_STATE),\
            mock.patch.object(self.fabric_base.dcnm_obj,
                              'create_partition') as create_part:
            self.fabric_base.fabric_fsm[fw_const.DCNM_OUT_NETWORK_STATE][0] = \
                mock.MagicMock()
            self.fabric_base.fabric_fsm[fw_const.DCNM_OUT_NETWORK_STATE][0].\
                return_value = False
            # update_fw_db is removed from here. Because both this and
            # update_fw_db_result uses the same fw_dict and fw_dict
            # is updated with result before calling update_fw_db_result. But,
            # mock records the updated fw_dict as a result of which argument
            # check fails.
            parent = mock.MagicMock()
            parent.attach_mock(create_part, 'create_partition')
            parent.attach_mock(self.upd_fw_db_res_mock, 'update_fw_db_result')
            parent.attach_mock(self.app_state_final_res_mock,
                               'append_state_final_result')
            self.fabric_base.prepare_fabric_fw(self.tenant_id,
                                               self.mock_fw_dict, True,
                                               fw_const.RESULT_FW_CREATE_INIT)
        expected_calls = [
            mock.call.create_partition(
                self.tenant_name, fw_const.SERV_PART_NAME, None,
                config.default_firewall_opts[
                    'firewall']['fw_service_part_vrf_profile'],
                desc='Service Partition'),
            mock.call.update_fw_db_result(self.mock_fw_dict.get('fw_id'),
                                          fw_dcnm_part_dict),
            mock.call.append_state_final_result(
                self.mock_fw_dict.get('fw_id'), fw_const.RESULT_FW_CREATE_INIT,
                fw_const.DCNM_OUT_NETWORK_STATE),
            mock.call.append_state_final_result(
                self.mock_fw_dict.get('fw_id'),
                fw_const.RESULT_FW_CREATE_INIT,
                fw_const.DCNM_OUT_NETWORK_STATE)]
        parent.assert_has_calls(expected_calls, any_order=False)

    def test_create_dcnm_out_nwk(self):
        """Create OUT Network Test.

        This function relies on the state information filled by previous
        functions. So, rather than starting fresh, we shall populate the
        FW DB. This is equivalent to restarting the enabler server and it
        reading the DB and populating the local cache.
        """
        fw_dcnm_out_net_dict = self._fill_fw_dcnm_out_net_dict()
        fw_dcnm_rest_out_net_dict = self._fill_fw_dcnm_rest_out_net_dict()
        fw_dcnm_rest_out_subnet_dict = (
            self._fill_fw_dcnm_rest_out_subnet_dict())
        fake_db_net = {'segmentation_id': self.out_segmentation_id,
                       'vlan': VLAN_ID}
        fake_net_obj = utils.Dict2Obj(fake_db_net)
        with mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                        'native.fabric_setup_base.ServiceIpSegTenantMap.'
                        'get_state',
                        return_value=fw_const.DCNM_OUT_NETWORK_STATE),\
            mock.patch('networking_cisco.apps.saf.common.utils.'
                       'Dict2Obj') as dict_obj,\
            mock.patch.object(self.fabric_base.dcnm_obj,
                              'create_service_network') as create_nwk:
            self.fabric_base.fabric_fsm[
                fw_const.DCNM_OUT_PART_UPDATE_STATE][0] = mock.MagicMock()
            self.fabric_base.fabric_fsm[
                fw_const.DCNM_OUT_PART_UPDATE_STATE][0].return_value = False
            # update_fw_db is removed from here. Because both this and
            # update_fw_db_result uses the same fw_dict and fw_dict
            # is updated with result before calling update_fw_db_result. But,
            # mock records the updated fw_dict as a result of which argument
            # check fails.
            FakeClass.set_return(
                dbm.DfaDBMixin, 'get_all_fw_db',
                self._fill_fw_db_data(fw_const.DCNM_OUT_NETWORK_STATE))
            FakeClass.set_return(dbm.DfaDBMixin, 'get_network', fake_net_obj)
            self.fabric_base.service_out_ip.get_subnet_by_netid.\
                return_value = self.out_subnet
            self.fabric_base.populate_local_cache()
            parent = mock.MagicMock()
            parent.attach_mock(dict_obj, 'Dict2Obj')
            parent.attach_mock(dict_obj, 'Dict2Obj')
            parent.attach_mock(self.upd_fw_db_res_mock, 'update_fw_db_result')
            parent.attach_mock(self.app_state_final_res_mock,
                               'append_state_final_result')
            self.fabric_base.prepare_fabric_fw(self.tenant_id,
                                               self.mock_fw_dict, True,
                                               fw_const.RESULT_FW_CREATE_INIT)
        expected_calls = [mock.call.Dict2Obj(fw_dcnm_rest_out_net_dict),
                          mock.call.Dict2Obj(fw_dcnm_rest_out_subnet_dict),
                          mock.call.update_fw_db_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_dcnm_out_net_dict),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_CREATE_INIT,
                              fw_const.DCNM_OUT_PART_UPDATE_STATE),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_CREATE_INIT,
                              fw_const.DCNM_OUT_PART_UPDATE_STATE)]
        parent.assert_has_calls(expected_calls, any_order=False)
        self.assertEqual(create_nwk.called, True)

    def test_create_dcnm_out_part_update(self):
        """DCNM Update OUT Partition Test. """
        fw_dcnm_part_dict = self._fill_fw_dcnm_part_upd_dict("out")
        with mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                        'native.fabric_setup_base.ServiceIpSegTenantMap.'
                        'get_state',
                        return_value=fw_const.DCNM_OUT_PART_UPDATE_STATE),\
            mock.patch.object(self.fabric_base.dcnm_obj,
                              'get_partition_segmentId',
                              return_value=self.ext_part),\
            mock.patch.object(self.fabric_base.dcnm_obj,
                              'update_project') as update_proj:
            # update_fw_db is removed from here. Because both this and
            # update_fw_db_result uses the same fw_dict and fw_dict
            # is updated with result before calling update_fw_db_result. But,
            # mock records the updated fw_dict as a result of which argument
            # check fails.
            parent = mock.MagicMock()
            parent.attach_mock(update_proj, 'update_project')
            parent.attach_mock(self.upd_fw_db_res_mock, 'update_fw_db_result')
            parent.attach_mock(self.app_state_final_res_mock,
                               'append_state_final_result')
            self.fabric_base.prepare_fabric_fw(self.tenant_id,
                                               self.mock_fw_dict, True,
                                               fw_const.RESULT_FW_CREATE_INIT)
        expected_calls = [
            mock.call.update_project(
                self.tenant_name, fw_const.SERV_PART_NAME,
                dci_id=self.ext_part,
                vrf_prof=config.default_firewall_opts[
                    'firewall']['fw_service_part_vrf_profile']),
            mock.call.update_fw_db_result(self.mock_fw_dict.get('fw_id'),
                                          fw_dcnm_part_dict),
            mock.call.append_state_final_result(
                self.mock_fw_dict.get('fw_id'), fw_const.RESULT_FW_CREATE_INIT,
                fw_const.FABRIC_PREPARE_DONE_STATE)]
        parent.assert_has_calls(expected_calls, any_order=False)

    def _test_delete_nwk(self, state, direc):
        fw_net_dict2 = self._fill_fw_del_net_dict()
        if direc == "in":
            net_id = self.net_id
            segmentation_id = self.segmentation_id
            subnet = self.in_subnet
            net_id = self.net_id
            fw_net_dict2['os_status'] = fw_const.OS_IN_NETWORK_DEL_SUCCESS
        else:
            net_id = self.out_net_id
            segmentation_id = self.out_segmentation_id
            subnet = self.out_subnet
            net_id = self.out_net_id
            fw_net_dict2['os_status'] = fw_const.OS_OUT_NETWORK_DEL_SUCCESS
        fw_db_data = self._fill_fw_db_data(state)
        fake_fw_obj = utils.Dict2Obj(fw_db_data.get(self.fw_id))
        with mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                        'native.fabric_setup_base.ServiceIpSegTenantMap.'
                        'get_state', return_value=state),\
            mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                       'native.fabric_setup_base.ServiceIpSegTenantMap.'
                       'fixup_state', return_value=state),\
            mock.patch.object(self.fabric_base.os_helper,
                              'delete_network_all_subnets',
                              return_value=True) as delete_nwk,\
            mock.patch.object(
                self.fabric_base.service_segs,
                'release_segmentation_id') as rel_seg, \
            mock.patch.object(
                self.fabric_base.service_in_ip, 'release_subnet') as rel_sub, \
            mock.patch.object(self.fabric_base,
                              'delete_network_db') as del_nwk_db:

            if direc == "in":
                FakeClass.set_return(fsb.FabricApi, 'get_in_ip_addr',
                                     {'subnet': self.in_subnet,
                                      'start': self.in_start,
                                      'sec_gateway': self.in_sec_gw,
                                      'gateway': self.in_gw,
                                      'end': self.in_end})
                self.fabric_base.fabric_fsm[fw_const.INIT_STATE][1] = \
                    mock.MagicMock()
                self.fabric_base.fabric_fsm[fw_const.INIT_STATE][1].\
                    return_value = False
            else:
                FakeClass.set_return(fsb.FabricApi, 'get_out_ip_addr',
                                     {'subnet': self.out_subnet,
                                      'start': self.out_start,
                                      'sec_gateway': self.out_sec_gw,
                                      'gateway': self.out_gw,
                                      'end': self.out_end})
                self.fabric_base.fabric_fsm[fw_const.OS_IN_NETWORK_STATE][1] = \
                    mock.MagicMock()
                self.fabric_base.fabric_fsm[fw_const.OS_IN_NETWORK_STATE][1].\
                    return_value = False
            # update_fw_db is removed from here. Because both this and
            # update_fw_db_result uses the same fw_dict and fw_dict
            # is updated with result before calling update_fw_db_result. But,
            # mock records the updated fw_dict as a result of which argument
            # check fails.
            FakeClass.set_return(dbm.DfaDBMixin, 'get_all_fw_db', fw_db_data)
            FakeClass.set_return(dbm.DfaDBMixin, 'get_fw', [fake_fw_obj, None])
            if direc == "in":
                FakeClass.set_return(fsb.FabricApi, 'get_in_seg_vlan',
                                     [self.segmentation_id, VLAN_ID])
                self.fabric_base.service_in_ip.get_subnet_by_netid.\
                    return_value = self.in_subnet
                prev_state = fw_const.INIT_STATE
            else:
                FakeClass.set_return(fsb.FabricApi, 'get_out_seg_vlan',
                                     [self.out_segmentation_id, VLAN_ID])
                self.fabric_base.service_out_ip.get_subnet_by_netid.\
                    return_value = self.out_subnet
                prev_state = fw_const.OS_IN_NETWORK_STATE
            self.fabric_base.populate_local_cache()
            parent = mock.MagicMock()
            parent.attach_mock(delete_nwk, 'delete_network_all_subnets')
            parent.attach_mock(del_nwk_db, 'delete_network_db')
            parent.attach_mock(self.upd_fw_db_res_mock, 'update_fw_db_result')
            parent.attach_mock(rel_seg, 'release_segmentation_id')
            parent.attach_mock(rel_sub, 'release_subnet')
            parent.attach_mock(self.app_state_final_res_mock,
                               'append_state_final_result')
            self.fabric_base.delete_fabric_fw(self.tenant_id,
                                              self.mock_fw_dict, True,
                                              fw_const.RESULT_FW_DELETE_INIT)
        expected_calls = [mock.call.delete_network_all_subnets(net_id),
                          mock.call.release_segmentation_id(segmentation_id),
                          mock.call.release_subnet(subnet),
                          mock.call.delete_network_db(net_id),
                          mock.call.update_fw_db_result(
                              self.mock_fw_dict.get('fw_id'), fw_net_dict2),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_DELETE_INIT, prev_state)]
        parent.assert_has_calls(expected_calls, any_order=False)

    def test_delete_in_nwk(self):
        """Delete IN Network Test. """
        self._test_delete_nwk(fw_const.OS_IN_NETWORK_STATE, "in")

    def _test_delete_nwk_fail(self, direc):
        """Internal function for delete network failure case.

        The Openstack delete network helper function is mocked to
        return a failure.
        """
        fw_net_dict2 = self._fill_fw_del_net_dict()
        if direc == "in":
            fw_net_dict2['os_status'] = fw_const.OS_IN_NETWORK_DEL_FAIL
            state = fw_const.OS_IN_NETWORK_STATE
            net_id = self.net_id
        else:
            fw_net_dict2['os_status'] = fw_const.OS_OUT_NETWORK_DEL_FAIL
            state = fw_const.OS_OUT_NETWORK_STATE
            net_id = self.out_net_id
        fw_db_data = self._fill_fw_db_data(state)
        fake_fw_obj = utils.Dict2Obj(fw_db_data.get(self.fw_id))
        with mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                        'native.fabric_setup_base.ServiceIpSegTenantMap.'
                        'get_state', return_value=state),\
            mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                       'native.fabric_setup_base.ServiceIpSegTenantMap.'
                       'fixup_state', return_value=state),\
            mock.patch.object(self.fabric_base.os_helper,
                              'delete_network_all_subnets',
                              return_value=False) as delete_nwk,\
            mock.patch.object(
                self.fabric_base.service_segs,
                'release_segmentation_id') as rel_seg, \
            mock.patch.object(
                self.fabric_base.service_in_ip, 'release_subnet') as rel_sub, \
            mock.patch.object(self.fabric_base,
                              'delete_network_db') as del_nwk_db:
            if direc == "in":
                FakeClass.set_return(fsb.FabricApi, 'get_in_ip_addr',
                                     {'subnet': self.in_subnet,
                                      'start': self.in_start,
                                      'sec_gateway': self.in_sec_gw,
                                      'gateway': self.in_gw,
                                      'end': self.in_end})
                self.fabric_base.fabric_fsm[fw_const.INIT_STATE][0] = \
                    mock.MagicMock()
                self.fabric_base.fabric_fsm[fw_const.INIT_STATE][0].\
                    return_value = False
                FakeClass.set_return(fsb.FabricApi, 'get_in_seg_vlan',
                                     [self.segmentation_id, VLAN_ID])
                self.fabric_base.service_in_ip.get_subnet_by_netid.\
                    return_value = self.in_subnet
            else:
                FakeClass.set_return(fsb.FabricApi, 'get_out_ip_addr',
                                     {'subnet': self.out_subnet,
                                      'start': self.out_start,
                                      'sec_gateway': self.out_sec_gw,
                                      'gateway': self.out_gw,
                                      'end': self.out_end})
                self.fabric_base.fabric_fsm[
                    fw_const.OS_IN_NETWORK_STATE][0] = mock.MagicMock()
                self.fabric_base.fabric_fsm[fw_const.OS_IN_NETWORK_STATE][0].\
                    return_value = False
                FakeClass.set_return(fsb.FabricApi, 'get_out_seg_vlan',
                                     [self.out_segmentation_id, VLAN_ID])
                self.fabric_base.service_out_ip.get_subnet_by_netid.\
                    return_value = self.out_subnet
            # update_fw_db is removed from here. Because both this and
            # update_fw_db_result uses the same fw_dict and fw_dict
            # is updated with result before calling update_fw_db_result. But,
            # mock records the updated fw_dict as a result of which argument
            # check fails.
            FakeClass.set_return(dbm.DfaDBMixin, 'get_all_fw_db', fw_db_data)
            FakeClass.set_return(dbm.DfaDBMixin, 'get_fw', [fake_fw_obj, None])
            self.fabric_base.populate_local_cache()
            parent = mock.MagicMock()
            parent.attach_mock(delete_nwk, 'delete_network_all_subnets')
            parent.attach_mock(del_nwk_db, 'delete_network_db')
            parent.attach_mock(self.upd_fw_db_res_mock, 'update_fw_db_result')
            parent.attach_mock(rel_seg, 'release_segmentation_id')
            parent.attach_mock(rel_sub, 'release_subnet')
            parent.attach_mock(self.app_state_final_res_mock,
                               'append_state_final_result')
            self.fabric_base.delete_fabric_fw(self.tenant_id,
                                              self.mock_fw_dict, True,
                                              fw_const.RESULT_FW_DELETE_INIT)
        expected_calls = [mock.call.delete_network_all_subnets(net_id),
                          mock.call.update_fw_db_result(
                              self.mock_fw_dict.get('fw_id'), fw_net_dict2),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_DELETE_INIT, state)]
        parent.assert_has_calls(expected_calls, any_order=False)
        rel_seg.assert_not_called()
        rel_sub.assert_not_called()
        del_nwk_db.assert_not_called()

    def test_delete_in_nwk_fail(self):
        """Delete IN Network Failure Test. """
        self._test_delete_nwk_fail("in")

    def test_delete_out_nwk(self):
        """Delete OUT Network Test. """
        self._test_delete_nwk(fw_const.OS_OUT_NETWORK_STATE, "out")

    def test_delete_out_nwk_fail(self):
        """Delete OUT Network Failure Test. """
        self._test_delete_nwk_fail("out")

    def test_delete_os_dummy_rtr(self):
        """Delete Dummy Router Test. """
        fw_net_dict2 = self._fill_fw_del_net_dict()
        fw_net_dict2['os_status'] = fw_const.OS_DUMMY_RTR_DEL_SUCCESS
        subnet_lst = set()
        subnet_lst.add(self.rtr_subnet_id)
        net_id = self.rtr_net_id

        fw_db_data = self._fill_fw_db_data(fw_const.OS_DUMMY_RTR_STATE)
        with mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                        'native.fabric_setup_base.ServiceIpSegTenantMap.'
                        'get_state',
                        return_value=fw_const.OS_DUMMY_RTR_STATE),\
            mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                       'native.fabric_setup_base.ServiceIpSegTenantMap.'
                       'fixup_state',
                       return_value=fw_const.OS_DUMMY_RTR_STATE),\
            mock.patch.object(self.fabric_base.os_helper,
                              'delete_intf_router',
                              return_value=True) as delete_intf_rtr,\
            mock.patch.object(self.fabric_base.os_helper,
                              'delete_network_all_subnets',
                              return_value=True) as delete_all_subnets,\
            mock.patch.object(self.fabric_base,
                              'delete_network_db') as del_nwk_db:
            self.fabric_base.fabric_fsm[fw_const.OS_OUT_NETWORK_STATE][1] = \
                mock.MagicMock()
            self.fabric_base.fabric_fsm[fw_const.OS_OUT_NETWORK_STATE][1].\
                return_value = False
            # update_fw_db is removed from here. Because both this and
            # update_fw_db_result uses the same fw_dict and fw_dict
            # is updated with result before calling update_fw_db_result. But,
            # mock records the updated fw_dict as a result of which argument
            # check fails.
            FakeClass.set_return(dbm.DfaDBMixin, 'get_all_fw_db', fw_db_data)
            self.fabric_base.service_in_ip.get_subnet_by_netid.\
                return_value = self.in_subnet
            self.fabric_base.populate_local_cache()

            parent = mock.MagicMock()
            parent.attach_mock(delete_intf_rtr, 'delete_intf_router')
            parent.attach_mock(delete_all_subnets,
                               'delete_network_all_subnets')
            parent.attach_mock(del_nwk_db, 'delete_network_db')
            parent.attach_mock(self.upd_fw_db_res_mock, 'update_fw_db_result')
            parent.attach_mock(self.app_state_final_res_mock,
                               'append_state_final_result')
            self.fabric_base.delete_fabric_fw(self.tenant_id,
                                              self.mock_fw_dict, False,
                                              fw_const.RESULT_FW_DELETE_INIT)
        expected_calls = [mock.call.delete_intf_router(None, None,
                                                       self.router_id,
                                                       subnet_lst),
                          mock.call.delete_network_all_subnets(net_id),
                          mock.call.delete_network_db(net_id),
                          mock.call.update_fw_db_result(
                              self.mock_fw_dict.get('fw_id'), fw_net_dict2),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_DELETE_INIT,
                              fw_const.OS_OUT_NETWORK_STATE),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_DELETE_INIT,
                              fw_const.OS_OUT_NETWORK_STATE)]
        parent.assert_has_calls(expected_calls, any_order=False)

    def test_delete_os_dummy_rtr_fail(self):
        """Delete Dummy Router Fail Test.

        The Openstack delete interface to router helper function is mocked to
        return a failure.
        """
        fw_net_dict2 = self._fill_fw_del_net_dict()
        fw_net_dict2['os_status'] = fw_const.OS_DUMMY_RTR_DEL_FAIL
        subnet_lst = set()
        subnet_lst.add(self.rtr_subnet_id)
        net_id = self.rtr_net_id

        fw_db_data = self._fill_fw_db_data(fw_const.OS_DUMMY_RTR_STATE)
        with mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                        'native.fabric_setup_base.ServiceIpSegTenantMap.'
                        'get_state',
                        return_value=fw_const.OS_DUMMY_RTR_STATE),\
            mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                       'native.fabric_setup_base.ServiceIpSegTenantMap.'
                       'fixup_state',
                       return_value=fw_const.OS_DUMMY_RTR_STATE),\
            mock.patch.object(self.fabric_base.os_helper,
                              'delete_intf_router',
                              return_value=False) as delete_intf_rtr,\
            mock.patch.object(self.fabric_base.os_helper,
                              'delete_network_all_subnets',
                              return_value=True) as delete_all_subnets,\
            mock.patch.object(self.fabric_base,
                              'delete_network_db') as del_nwk_db:
            # update_fw_db is removed from here. Because both this and
            # update_fw_db_result uses the same fw_dict and fw_dict
            # is updated with result before calling update_fw_db_result. But,
            # mock records the updated fw_dict as a result of which argument
            # check fails.
            FakeClass.set_return(dbm.DfaDBMixin, 'get_all_fw_db', fw_db_data)
            self.fabric_base.service_in_ip.get_subnet_by_netid.\
                return_value = self.in_subnet
            self.fabric_base.populate_local_cache()

            parent = mock.MagicMock()
            parent.attach_mock(delete_intf_rtr, 'delete_intf_router')
            parent.attach_mock(delete_all_subnets,
                               'delete_network_all_subnets')
            parent.attach_mock(del_nwk_db, 'delete_network_db')
            parent.attach_mock(self.upd_fw_db_res_mock, 'update_fw_db_result')
            parent.attach_mock(self.app_state_final_res_mock,
                               'append_state_final_result')
            self.fabric_base.delete_fabric_fw(self.tenant_id,
                                              self.mock_fw_dict, False,
                                              fw_const.RESULT_FW_DELETE_INIT)
        expected_calls = [mock.call.delete_intf_router(None, None,
                                                       self.router_id,
                                                       subnet_lst),
                          mock.call.delete_network_db(net_id),
                          mock.call.update_fw_db_result(
                              self.mock_fw_dict.get('fw_id'), fw_net_dict2),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_DELETE_INIT,
                              fw_const.OS_DUMMY_RTR_STATE)]
        parent.assert_has_calls(expected_calls, any_order=False)
        delete_all_subnets.assert_not_called()

    def _test_delete_dcnm_nwk(self, direc):
        fw_net_dict2 = self._fill_fw_del_net_dict()
        if direc == "in":
            fw_net_dict2['dcnm_status'] = fw_const.DCNM_IN_NETWORK_DEL_SUCCESS
            state = fw_const.DCNM_IN_NETWORK_STATE
            prev_state = fw_const.OS_DUMMY_RTR_STATE
            part = None
        else:
            fw_net_dict2['dcnm_status'] = fw_const.DCNM_OUT_NETWORK_DEL_SUCCESS
            state = fw_const.DCNM_OUT_NETWORK_STATE
            prev_state = fw_const.DCNM_OUT_PART_STATE
            part = fw_const.SERV_PART_NAME
        fw_net_dict2['os_status'] = fw_const.OS_DUMMY_RTR_CREATE_SUCCESS

        fw_db_data = self._fill_fw_db_data(state)
        with mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                        'native.fabric_setup_base.ServiceIpSegTenantMap.'
                        'get_state', return_value=state),\
            mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                       'native.fabric_setup_base.ServiceIpSegTenantMap.'
                       'fixup_state', return_value=state),\
            mock.patch('networking_cisco.apps.saf.common.utils.'
                       'Dict2Obj') as dict_obj,\
            mock.patch.object(self.fabric_base.dcnm_obj,
                              'delete_service_network') as delete_nwk:
            self.fabric_base.fabric_fsm[prev_state][1] = mock.MagicMock()
            self.fabric_base.fabric_fsm[prev_state][1].return_value = False
            # update_fw_db is removed from here. Because both this and
            # update_fw_db_result uses the same fw_dict and fw_dict
            # is updated with result before calling update_fw_db_result. But,
            # mock records the updated fw_dict as a result of which argument
            # check fails.
            FakeClass.set_return(dbm.DfaDBMixin, 'get_all_fw_db', fw_db_data)
            if direc == "in":
                FakeClass.set_return(fsb.FabricApi, 'get_in_seg_vlan',
                                     [self.segmentation_id, VLAN_ID])
                fake_db_net = {'part_name': None,
                               'segmentation_id': self.segmentation_id,
                               'vlan': VLAN_ID}
                self.fabric_base.service_in_ip.get_subnet_by_netid.\
                    return_value = self.in_subnet
            else:
                FakeClass.set_return(fsb.FabricApi, 'get_out_seg_vlan',
                                     [self.out_segmentation_id, VLAN_ID])
                fake_db_net = {'part_name': part,
                               'segmentation_id': self.out_segmentation_id,
                               'vlan': VLAN_ID}
                self.fabric_base.service_out_ip.get_subnet_by_netid.\
                    return_value = self.out_subnet
            self.fabric_base.populate_local_cache()

            parent = mock.MagicMock()
            parent.attach_mock(self.upd_fw_db_res_mock, 'update_fw_db_result')
            parent.attach_mock(self.app_state_final_res_mock,
                               'append_state_final_result')
            parent.attach_mock(dict_obj, 'Dict2Obj')
            self.fabric_base.delete_fabric_fw(self.tenant_id,
                                              self.mock_fw_dict, False,
                                              fw_const.RESULT_FW_DELETE_INIT)
        expected_calls = [mock.call.Dict2Obj(fake_db_net),
                          mock.call.update_fw_db_result(
                              self.mock_fw_dict.get('fw_id'), fw_net_dict2),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_DELETE_INIT, prev_state),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_DELETE_INIT, prev_state)]
        parent.assert_has_calls(expected_calls, any_order=False)
        self.assertEqual(delete_nwk.called, True)

    def test_delete_dcnm_in_nwk(self):
        """Delete DCNM In Network Test. """
        self._test_delete_dcnm_nwk("in")

    def test_clear_dcnm_in_part(self):
        """Clear DCNM IN partition service node address Test. """
        fw_net_dict2 = self._fill_fw_del_net_dict()
        fw_net_dict2['dcnm_status'] = fw_const.DCNM_IN_PART_UPDDEL_SUCCESS
        fw_net_dict2['os_status'] = fw_const.OS_DUMMY_RTR_CREATE_SUCCESS

        fw_db_data = self._fill_fw_db_data(fw_const.DCNM_IN_PART_UPDATE_STATE)
        with mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                        'native.fabric_setup_base.ServiceIpSegTenantMap.'
                        'get_state',
                        return_value=fw_const.DCNM_IN_PART_UPDATE_STATE),\
            mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                       'native.fabric_setup_base.ServiceIpSegTenantMap.'
                       'fixup_state',
                       return_value=fw_const.DCNM_IN_PART_UPDATE_STATE),\
            mock.patch.object(self.fabric_base.dcnm_obj,
                              'update_project') as update_project:
            self.fabric_base.fabric_fsm[
                fw_const.DCNM_IN_NETWORK_STATE][1] = mock.MagicMock()
            self.fabric_base.fabric_fsm[
                fw_const.DCNM_IN_NETWORK_STATE][1].return_value = False
            # update_fw_db is removed from here. Because both this and
            # update_fw_db_result uses the same fw_dict and fw_dict
            # is updated with result before calling update_fw_db_result. But,
            # mock records the updated fw_dict as a result of which argument
            # check fails.
            FakeClass.set_return(dbm.DfaDBMixin, 'get_all_fw_db', fw_db_data)
            FakeClass.set_return(fsb.FabricApi, 'get_in_ip_addr',
                                 {'subnet': self.in_subnet,
                                  'start': self.in_start,
                                  'sec_gateway': self.in_sec_gw,
                                  'gateway': self.in_gw,
                                  'end': self.in_end})
            self.fabric_base.service_in_ip.get_subnet_by_netid.\
                return_value = self.in_subnet
            self.fabric_base.populate_local_cache()

            parent = mock.MagicMock()
            parent.attach_mock(update_project, 'update_project')
            parent.attach_mock(self.upd_fw_db_res_mock, 'update_fw_db_result')
            parent.attach_mock(self.app_state_final_res_mock,
                               'append_state_final_result')
            self.fabric_base.delete_fabric_fw(self.tenant_id,
                                              self.mock_fw_dict, False,
                                              fw_const.RESULT_FW_DELETE_INIT)
        expected_calls = [mock.call.update_project(self.tenant_name, None,
                                                   service_node_ip=None,
                                                   vrf_prof=None,
                                                   desc="Service Partition"),
                          mock.call.update_fw_db_result(
                              self.mock_fw_dict.get('fw_id'), fw_net_dict2),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_DELETE_INIT,
                              fw_const.DCNM_IN_NETWORK_STATE),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_DELETE_INIT,
                              fw_const.DCNM_IN_NETWORK_STATE)]
        parent.assert_has_calls(expected_calls, any_order=False)

    def test_delete_dcnm_out_part(self):
        """Delete DCNM OUT partition address Test. """
        fw_net_dict2 = self._fill_fw_del_net_dict()
        fw_net_dict2['dcnm_status'] = fw_const.DCNM_OUT_PART_DEL_SUCCESS
        fw_net_dict2['os_status'] = fw_const.OS_DUMMY_RTR_CREATE_SUCCESS

        fw_db_data = self._fill_fw_db_data(fw_const.DCNM_OUT_PART_STATE)
        with mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                        'native.fabric_setup_base.ServiceIpSegTenantMap.'
                        'get_state',
                        return_value=fw_const.DCNM_OUT_PART_STATE),\
            mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                       'native.fabric_setup_base.ServiceIpSegTenantMap.'
                       'fixup_state',
                       return_value=fw_const.DCNM_OUT_PART_STATE),\
            mock.patch.object(self.fabric_base.dcnm_obj,
                              'delete_partition') as delete_partition:
            self.fabric_base.fabric_fsm[
                fw_const.DCNM_IN_PART_UPDATE_STATE][1] = mock.MagicMock()
            self.fabric_base.fabric_fsm[
                fw_const.DCNM_IN_PART_UPDATE_STATE][1].return_value = False
            # update_fw_db is removed from here. Because both this and
            # update_fw_db_result uses the same fw_dict and fw_dict
            # is updated with result before calling update_fw_db_result. But,
            # mock records the updated fw_dict as a result of which argument
            # check fails.
            FakeClass.set_return(dbm.DfaDBMixin, 'get_all_fw_db', fw_db_data)
            self.fabric_base.service_in_ip.get_subnet_by_netid.\
                return_value = self.in_subnet
            self.fabric_base.populate_local_cache()

            parent = mock.MagicMock()
            parent.attach_mock(delete_partition, 'delete_partition')
            parent.attach_mock(self.upd_fw_db_res_mock, 'update_fw_db_result')
            parent.attach_mock(self.app_state_final_res_mock,
                               'append_state_final_result')
            self.fabric_base.delete_fabric_fw(self.tenant_id,
                                              self.mock_fw_dict, False,
                                              fw_const.RESULT_FW_DELETE_INIT)
        expected_calls = [mock.call.delete_partition(self.tenant_name,
                                                     fw_const.SERV_PART_NAME),
                          mock.call.update_fw_db_result(
                              self.mock_fw_dict.get('fw_id'), fw_net_dict2),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_DELETE_INIT,
                              fw_const.DCNM_IN_PART_UPDATE_STATE),
                          mock.call.append_state_final_result(
                              self.mock_fw_dict.get('fw_id'),
                              fw_const.RESULT_FW_DELETE_INIT,
                              fw_const.DCNM_IN_PART_UPDATE_STATE)]
        parent.assert_has_calls(expected_calls, any_order=False)

    def test_delete_dcnm_out_nwk(self):
        """Delete DCNM Out Network Test. """
        self._test_delete_dcnm_nwk("out")

    def test_clear_dcnm_out_part(self):
        """Clear DCNM OUT partition service node address Test. """
        fw_net_dict2 = self._fill_fw_del_net_dict()
        fw_net_dict2['dcnm_status'] = fw_const.DCNM_OUT_PART_UPDDEL_SUCCESS
        fw_net_dict2['os_status'] = fw_const.OS_DUMMY_RTR_CREATE_SUCCESS

        fw_db_data = self._fill_fw_db_data(fw_const.DCNM_OUT_PART_UPDATE_STATE)
        with mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                        'native.fabric_setup_base.ServiceIpSegTenantMap.'
                        'get_state',
                        return_value=fw_const.DCNM_OUT_PART_UPDATE_STATE),\
            mock.patch('networking_cisco.apps.saf.server.services.firewall.'
                       'native.fabric_setup_base.ServiceIpSegTenantMap.'
                       'fixup_state',
                       return_value=fw_const.DCNM_OUT_PART_UPDATE_STATE):
            self.fabric_base.fabric_fsm[
                fw_const.DCNM_OUT_NETWORK_STATE][1] = mock.MagicMock()
            self.fabric_base.fabric_fsm[
                fw_const.DCNM_OUT_NETWORK_STATE][1].return_value = False
            # update_fw_db is removed from here. Because both this and
            # update_fw_db_result uses the same fw_dict and fw_dict
            # is updated with result before calling update_fw_db_result. But,
            # mock records the updated fw_dict as a result of which argument
            # check fails.
            FakeClass.set_return(dbm.DfaDBMixin, 'get_all_fw_db', fw_db_data)
            self.fabric_base.service_in_ip.get_subnet_by_netid.\
                return_value = self.in_subnet
            self.fabric_base.populate_local_cache()

            parent = mock.MagicMock()
            parent.attach_mock(self.upd_fw_db_res_mock, 'update_fw_db_result')
            parent.attach_mock(self.app_state_final_res_mock,
                               'append_state_final_result')
            self.fabric_base.delete_fabric_fw(self.tenant_id,
                                              self.mock_fw_dict, False,
                                              fw_const.RESULT_FW_DELETE_INIT)
        expected_calls = [
            mock.call.update_fw_db_result(self.mock_fw_dict.get('fw_id'),
                                          fw_net_dict2),
            mock.call.append_state_final_result(
                self.mock_fw_dict.get('fw_id'), fw_const.RESULT_FW_DELETE_INIT,
                fw_const.DCNM_OUT_NETWORK_STATE),
            mock.call.append_state_final_result(
                self.mock_fw_dict.get('fw_id'), fw_const.RESULT_FW_DELETE_INIT,
                fw_const.DCNM_OUT_NETWORK_STATE)]
        parent.assert_has_calls(expected_calls, any_order=False)



# Copyright 2014 Cisco Systems, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from networking_cisco import backwards_compatibility as bc
import networking_cisco.plugins.cisco.device_manager.plugging_drivers as plug
from networking_cisco.plugins.cisco.device_manager.plugging_drivers import (
    utils)


class NoopPluggingDriver(plug.PluginSidePluggingDriver,
                         utils.PluggingDriverUtilsMixin):
    """This class defines a no-op plugging driver."""

    @property
    def _core_plugin(self):
        return bc.get_plugin()

    def create_hosting_device_resources(self, context, complementary_id,
                                        tenant_id, mgmt_context, max_hosted):
        return {'mgmt_port': None, 'ports': []}

    def get_hosting_device_resources(self, context, id, complementary_id,
                                     tenant_id, mgmt_nw_id):
        return {'mgmt_port': None, 'ports': []}

    def delete_hosting_device_resources(self, context, tenant_id, mgmt_port,
                                        **kwargs):
        pass

    def setup_logical_port_connectivity(self, context, port_db,
                                        hosting_device_id):
        pass

    def teardown_logical_port_connectivity(self, context, port_db,
                                        hosting_device_id):
        pass

    def extend_hosting_port_info(self, context, port_db, hosting_device,
                                 hosting_info):
        pass

    def allocate_hosting_port(self, context, router_id, port_db, network_type,
                              hosting_device_id):
        pass



# Copyright 2015 Cisco Systems, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.


from networking_cisco import backwards_compatibility as bc


ROUTERROLE = 'routerrole'
ROUTERROLE_ALIAS = ROUTERROLE
ROUTER_ROLE_ATTR = ROUTERROLE + ':role'

EXTENDED_ATTRIBUTES_2_0 = {
    'routers': {
        ROUTER_ROLE_ATTR: {'allow_post': False, 'allow_put': False,
                           'default': bc.constants.ATTR_NOT_SPECIFIED,
                           'is_visible': True},
    }
}


class Routerrole(bc.extensions.ExtensionDescriptor):
    """Extension class to introduce role information for routers.

    This class is used by Neutron's extension framework to add role attribute
    to Neutron Routers implemented in virtual/physical appliances.
    """

    @classmethod
    def get_name(cls):
        return "Role info for routing service"

    @classmethod
    def get_alias(cls):
        return ROUTERROLE_ALIAS

    @classmethod
    def get_description(cls):
        return ("Introduces role attribute for Neutron routers "
                "implemented in virtual/physical appliances")

    @classmethod
    def get_namespace(cls):
        return ("http://docs.openstack.org/ext/" + ROUTERROLE + "/api/v1.0")

    @classmethod
    def get_updated(cls):
        return "2015-02-07T10:00:00-00:00"

    def get_extended_resources(self, version):
        if version == "2.0":
            return EXTENDED_ATTRIBUTES_2_0
        else:
            return {}



# Copyright 2015 Cisco Systems, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import logging

from networking_cisco.plugins.cisco.cfg_agent.device_drivers.asr1k import (
    asr1k_routing_driver as asr_driver)
from networking_cisco.plugins.cisco.cfg_agent.service_helpers.routing_svc_helper \
    import RouterInfo

if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    device_params = {'management_ip_address': '10.86.7.178',
                     'protocol_port': 22,
                     'timeout': 30,
                     "credentials": {
                         'username': "stack",
                         'password': 'cisco'
                     }}
    router = {'id': 'dummy',
              'name': 'router1'}
    ri = RouterInfo("dummy", router)
    port = {'ip_cidr': '10.0.2.1/24',
            'hosting_info': {
                'hosting_mac': 'fa:16:3e:b3:0e:48',
                'hosting_port_id': 'dummy_id',
                'hosting_port_name': u't2_p:1',
                'segmentation_id': 101,
                'physical_interface': 'GigabitEthernet0/0/0'}
            }
    driver = asr_driver.ASR1kRoutingDriver(**device_params)
    if driver._get_connection():
        logging.info('Connection Established!')
        driver.router_added(ri)
        driver.internal_network_added(ri, port)
        driver.internal_network_removed(ri, port)
        driver.router_removed(ri)



# Copyright 2017 Cisco Systems, Inc.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""update_ha_group_primary_key

Revision ID: 73c84db9f299
Revises: 972479e0e629
Create Date: 2017-10-05 05:31:54.243849

"""

# revision identifiers, used by Alembic.
revision = '73c84db9f299'
down_revision = '972479e0e629'

from alembic import op
from sqlalchemy.engine import reflection

from neutron.db import migration


def upgrade():
    if migration.schema_has_table('cisco_router_ha_groups'):
        inspector = reflection.Inspector.from_engine(op.get_bind())
        foreign_keys = inspector.get_foreign_keys('cisco_router_ha_groups')
        migration.remove_foreign_keys('cisco_router_ha_groups', foreign_keys)
        primary_key = inspector.get_pk_constraint('cisco_router_ha_groups')
        op.drop_constraint(constraint_name=primary_key['name'],
                           table_name='cisco_router_ha_groups',
                           type_='primary')

        op.create_primary_key(
            constraint_name='pk_cisco_router_ha_groups',
            table_name='cisco_router_ha_groups',
            columns=['ha_port_id', 'subnet_id'])

        op.create_foreign_key('cisco_router_ha_groups_ibfk_1',
                              source_table='cisco_router_ha_groups',
                              referent_table='ports',
                              local_cols=['ha_port_id'],
                              remote_cols=['id'],
                              ondelete='CASCADE'),
        op.create_foreign_key('cisco_router_ha_groups_ibfk_2',
                              source_table='cisco_router_ha_groups',
                              referent_table='ports',
                              local_cols=['extra_port_id'],
                              remote_cols=['id'],
                              ondelete='SET NULL'),
        op.create_foreign_key('cisco_router_ha_groups_ibfk_3',
                              source_table='cisco_router_ha_groups',
                              referent_table='subnets',
                              local_cols=['subnet_id'],
                              remote_cols=['id'])
        op.create_foreign_key('cisco_router_ha_groups_ibfk_4',
                              source_table='cisco_router_ha_groups',
                              referent_table='routers',
                              local_cols=['user_router_id'],
                              remote_cols=['id'])



# Copyright 2015-2016 Cisco Systems, Inc.
# All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import warnings

from oslo_config import cfg
from oslo_log import log as logging
from oslo_utils import importutils

from networking_cisco.backwards_compatibility import cb_constants as p_const
from networking_cisco.backwards_compatibility import ml2_api as api

from networking_cisco import backwards_compatibility as bc
from networking_cisco.ml2_drivers.ucsm import config
from networking_cisco.ml2_drivers.ucsm import constants as const
from networking_cisco.ml2_drivers.ucsm import deprecated_network_driver
from networking_cisco.ml2_drivers.ucsm import ucsm_db
from networking_cisco.ml2_drivers.ucsm import ucsm_network_driver

LOG = logging.getLogger(__name__)
CONF = cfg.CONF


class CiscoUcsmMechanismDriver(api.MechanismDriver):

    """ML2 Mechanism Driver for Cisco UCS Manager."""

    def initialize(self):
        self.vif_type = const.VIF_TYPE_802_QBH
        self.vif_details = {bc.portbindings.CAP_PORT_FILTER: False}
        self.ucsm_db = ucsm_db.UcsmDbModel()

        if importutils.try_import('ucsmsdk'):
            self.driver = ucsm_network_driver.CiscoUcsmDriver()
        elif importutils.try_import('UcsSdk'):
            warnings.warn("Cannot find ucsmsdk package, falling back to "
                    "using UcsSdk which is now deprecated. For new "
                    "features please run with ucsmsdk installed.",
                    DeprecationWarning)
            self.driver = deprecated_network_driver.CiscoUcsmDriver()
        else:
            LOG.error('Could not import ucsm sdk.')

        config.load_single_ucsm_config()

    def _get_vlanid(self, context):
        """Returns vlan_id associated with a bound VLAN segment."""
        segment = context.bottom_bound_segment
        if segment and self.check_segment(segment):
            return segment.get(api.SEGMENTATION_ID)

    def _is_supported_deviceowner(self, port):
        return (port['device_owner'].startswith('compute') or
                port['device_owner'] in [
                    bc.constants.DEVICE_OWNER_DHCP,
                    bc.constants.DEVICE_OWNER_ROUTER_HA_INTF])

    def _is_status_active(self, port):
        return port['status'] == bc.constants.PORT_STATUS_ACTIVE

    def _get_physnet(self, context):
        """Returns physnet associated with a bound VLAN segment."""
        segment = context.bottom_bound_segment
        if segment and self.check_segment(segment):
            return segment.get(api.PHYSICAL_NETWORK)

    def update_port_precommit(self, context):
        """Adds port profile and vlan information to the DB.

        Assign a port profile to this port. To do that:
        1. Get the vlan_id associated with the bound segment
        2. Check if a port profile already exists for this vlan_id
        3. If yes, associate that port profile with this port.
        4. If no, create a new port profile with this vlan_id and
        associate with this port
        """
        vnic_type = context.current.get(bc.portbindings.VNIC_TYPE,
                                        bc.portbindings.VNIC_NORMAL)

        profile = context.current.get(bc.portbindings.PROFILE, {})
        host_id = self._get_host_id(
            context.current.get(bc.portbindings.HOST_ID))
        if not host_id:
            LOG.warning('Host id from port context is None. '
                'Ignoring this port')
            return

        vlan_id = self._get_vlanid(context)
        if not vlan_id:
            LOG.warning('Vlan_id is None. Ignoring this port')
            return

        ucsm_ip = self.driver.get_ucsm_ip_for_host(host_id)
        if not ucsm_ip:
            LOG.info('Host %s is not controlled by any known '
                     'UCS Manager.', host_id)
            return

        if not self.driver.check_vnic_type_and_vendor_info(vnic_type,
                                                           profile):
            # This is a neutron virtio port.
            # If VNIC templates are configured, that config would
            # take precedence and the VLAN is added to the VNIC template.
            physnet = self._get_physnet(context)
            if not physnet:
                LOG.debug('physnet is None. Not modifying VNIC '
                          'Template config')
            else:
                # Check if VNIC template is configured for this physnet
                ucsm = CONF.ml2_cisco_ucsm.ucsms[ucsm_ip]
                vnic_template = ucsm.vnic_template_list.get(physnet)
                if vnic_template:
                    LOG.debug('vnic_template %s', vnic_template)
                    self.ucsm_db.add_vnic_template(vlan_id, ucsm_ip,
                        vnic_template.name, physnet)
                    return
                else:
                    LOG.debug('VNIC Template not configured for '
                              'physnet %s', physnet)

            # In the absence of VNIC Templates, VLAN is directly added
            # to vNIC(s) on the SP Template.
            # Check if SP Template config has been provided. If so, find
            # the UCSM that controls this host and the Service Profile
            # Template for this host.

            sp_template_info = (CONF.ml2_cisco_ucsm.ucsms[
                ucsm_ip].sp_template_list.get(host_id))

            if sp_template_info:
                LOG.debug('SP Template: %s, VLAN_id: %d',
                          sp_template_info.name, vlan_id)
                self.ucsm_db.add_service_profile_template(
                    vlan_id, sp_template_info.name, ucsm_ip)
                return

        # If this is an Intel SR-IOV vnic, then no need to create port
        # profile on the UCS manager. So no need to update the DB.
        if not self.driver.is_vmfex_port(profile):
            LOG.debug('This is a SR-IOV port and hence not updating DB.')
            return

        # This is a Cisco VM-FEX port
        p_profile_name = self.make_profile_name(vlan_id)
        LOG.debug('Port Profile: %s for VLAN_id: %d', p_profile_name, vlan_id)

        # Create a new port profile entry in the db
        self.ucsm_db.add_port_profile(p_profile_name, vlan_id, ucsm_ip)

    def update_port_postcommit(self, context):
        """Creates a port profile on UCS Manager.

        Creates a Port Profile for this VLAN if it does not already
        exist.
        """
        vlan_id = self._get_vlanid(context)

        if not vlan_id:
            LOG.warning('Vlan_id is None. Ignoring this port.')
            return

        if (not self._is_supported_deviceowner(context.current) or
            not self._is_status_active(context.current)):
            LOG.debug("Unsupported device_owner '%(owner)s' or port not"
                      " active (vlan_id '%(vlan)d', status %(status)s)."
                      "   Nothing to do.",
                      {'owner': context.current['device_owner'],
                       'vlan': vlan_id,
                       'status': context.current['status']})
            return

        # Checks to perform before UCS Manager can create a Port Profile.
        # 1. Make sure this host is on a known UCS Manager.
        host_id = self._get_host_id(
            context.current.get(bc.portbindings.HOST_ID))
        if not host_id:
            LOG.warning('Host id from port context is None. '
                'Ignoring this port')
            return

        ucsm_ip = self.driver.get_ucsm_ip_for_host(host_id)
        if not ucsm_ip:
            LOG.info('Host_id %s is not controlled by any known UCS '
                'Manager', str(host_id))
            return

        profile = context.current.get(bc.portbindings.PROFILE, {})
        vnic_type = context.current.get(bc.portbindings.VNIC_TYPE,
                                        bc.portbindings.VNIC_NORMAL)

        # 2. Make sure this is a vm_fex_port.(Port profiles are created
        # only for VM-FEX ports.)
        if (self.driver.check_vnic_type_and_vendor_info(vnic_type, profile) and
            self.driver.is_vmfex_port(profile)):

            # 3. Make sure update_port_precommit added an entry in the DB
            # for this port profile
            profile_name = self.ucsm_db.get_port_profile_for_vlan(vlan_id,
                ucsm_ip)

            # 4. Make sure that the Port Profile hasn't already been created
            # on the UCS Manager
            if profile_name and self.ucsm_db.is_port_profile_created(vlan_id,
                ucsm_ip):
                LOG.debug('Port Profile %s for vlan_id %d already exists '
                          'on UCSM %s.', profile_name, vlan_id, ucsm_ip)
                return

            # Multi VLAN trunk support
            # Check if this network is a trunk network. If so pass the
            # additional VLAN ids to the UCSM driver.
            network = context.network.current['name']
            trunk_vlans = (
                CONF.sriov_multivlan_trunk.network_vlans.get(network, []))

            # All checks are done. Ask the UCS Manager driver to create the
            # above Port Profile.
            if self.driver.create_portprofile(profile_name, vlan_id,
                                              vnic_type, host_id, trunk_vlans):
                # Port profile created on UCS, record that in the DB.
                self.ucsm_db.set_port_profile_created(vlan_id, profile_name,
                    ucsm_ip)
            return
        else:
            # Enable vlan-id for this Neutron virtual port.
            LOG.debug('Host_id is %s', host_id)
            physnet = self._get_physnet(context)

            ucsm = CONF.ml2_cisco_ucsm.ucsms[ucsm_ip]
            vnic_template = ucsm.vnic_template_list.get(physnet)

            if vnic_template:
                LOG.debug('Update VNIC Template for physnet: %s', physnet)
                LOG.debug('vnic_template %s', vnic_template)
                if (self.driver.update_vnic_template(
                        host_id, vlan_id, physnet, vnic_template.path,
                        vnic_template.name)):
                    LOG.debug('Setting ucsm_updated flag for '
                              'vlan : %(vlan)d, '
                              'vnic_template : %(vnic_template)s '
                              'on ucsm_ip: %(ucsm_ip)s',
                              {'vlan': vlan_id,
                              'vnic_template': vnic_template.name,
                              'ucsm_ip': ucsm_ip})
                    self.ucsm_db.set_vnic_template_updated(
                        vlan_id, ucsm_ip, vnic_template.name, physnet)
                return

            if (CONF.ml2_cisco_ucsm.ucsms[ucsm_ip].sp_template_list
                and self.driver.update_service_profile_template(
                    vlan_id, host_id, ucsm_ip)):
                sp_template_info = (CONF.ml2_cisco_ucsm.ucsms[
                    ucsm_ip].sp_template_list.get(host_id))
                if not sp_template_info:
                    sp_template = None
                else:
                    sp_template = sp_template_info.name
                LOG.debug('Setting ucsm_updated flag for vlan : %(vlan)d, '
                          'sp_template : %(sp_template)s on ucsm_ip: '
                          '%(ucsm_ip)s', {'vlan': vlan_id,
                          'sp_template': sp_template, 'ucsm_ip': ucsm_ip})
                self.ucsm_db.set_sp_template_updated(vlan_id, sp_template,
                                                     ucsm_ip)
            else:
                self.driver.update_serviceprofile(host_id, vlan_id)

    def delete_network_precommit(self, context):
        """Delete entry corresponding to Network's VLAN in the DB."""
        segments = context.network_segments
        for segment in segments:
            if not self.check_segment(segment):
                return  # Not a vlan network
            vlan_id = segment.get(api.SEGMENTATION_ID)
            if not vlan_id:
                return  # No vlan assigned to segment

            # For VM-FEX ports
            self.ucsm_db.delete_vlan_entry(vlan_id)
            # For Neutron virtio ports
            if any([True for ip, ucsm in CONF.ml2_cisco_ucsm.ucsms.items()
                    if ucsm.sp_template_list]):
                # At least on UCSM has sp templates configured
                self.ucsm_db.delete_sp_template_for_vlan(vlan_id)

            if any([True for ip, ucsm in CONF.ml2_cisco_ucsm.ucsms.items()
                    if ucsm.vnic_template_list]):
                # At least one UCSM has vnic templates configured
                self.ucsm_db.delete_vnic_template_for_vlan(vlan_id)

    def delete_network_postcommit(self, context):
        """Delete all configuration added to UCS Manager for the vlan_id."""
        segments = context.network_segments
        network_name = context.current['name']

        for segment in segments:
            if not self.check_segment(segment):
                return  # Not a vlan network
            vlan_id = segment.get(api.SEGMENTATION_ID)
            if not vlan_id:
                return  # No vlan assigned to segment

            port_profile = self.make_profile_name(vlan_id)
            trunk_vlans = (
                CONF.sriov_multivlan_trunk.network_vlans.get(network_name, []))
            self.driver.delete_all_config_for_vlan(vlan_id, port_profile,
                trunk_vlans)

    def bind_port(self, context):
        """Binds port to current network segment.

        Binds port only if the vnic_type is direct or macvtap and
        the port is from a supported vendor. While binding port set it
        in ACTIVE state and provide the Port Profile or Vlan Id as part
        vif_details.
        """
        vnic_type = context.current.get(bc.portbindings.VNIC_TYPE,
                                        bc.portbindings.VNIC_NORMAL)

        LOG.debug('Attempting to bind port %(port)s with vnic_type '
                  '%(vnic_type)s on network %(network)s ',
                  {'port': context.current['id'],
                   'vnic_type': vnic_type,
                   'network': context.network.current['id']})

        profile = context.current.get(bc.portbindings.PROFILE, {})

        if not self.driver.check_vnic_type_and_vendor_info(vnic_type,
                                                           profile):
            return

        for segment in context.network.network_segments:
            if self.check_segment(segment):
                vlan_id = segment[api.SEGMENTATION_ID]

                if not vlan_id:
                    LOG.warning('Cannot bind port: vlan_id is None.')
                    return

                LOG.debug("Port binding to Vlan_id: %s", str(vlan_id))

                # Check if this is a Cisco VM-FEX port or Intel SR_IOV port
                if self.driver.is_vmfex_port(profile):
                    profile_name = self.make_profile_name(vlan_id)
                    self.vif_details[
                        const.VIF_DETAILS_PROFILEID] = profile_name
                else:
                    self.vif_details[
                        bc.portbindings.VIF_DETAILS_VLAN] = str(vlan_id)

                context.set_binding(segment[api.ID],
                                    self.vif_type,
                                    self.vif_details,
                                    bc.constants.PORT_STATUS_ACTIVE)
                return

        LOG.error('UCS Mech Driver: Failed binding port ID %(id)s '
                  'on any segment of network %(network)s',
                  {'id': context.current['id'],
                   'network': context.network.current['id']})

    @staticmethod
    def check_segment(segment):
        network_type = segment[api.NETWORK_TYPE]
        return network_type == p_const.TYPE_VLAN

    @staticmethod
    def make_profile_name(vlan_id):
        return const.PORT_PROFILE_NAME_PREFIX + str(vlan_id)

    def _get_host_id(self, host_id):
        """Strips the host_id of any domain name extensions."""
        return host_id.split('.')[0] if host_id else None



# Copyright 2014 Cisco Systems, Inc.  All rights reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import contextlib
import mock
import os
import unittest

from neutron.api.v2 import attributes
from neutron.db import agents_db
from neutron.db import dns_db
from neutron.extensions import providernet as pnet
from neutron.tests.unit.db import test_db_base_plugin_v2
from neutron.tests.unit.extensions import test_extraroute
from neutron.tests.unit.extensions import test_l3
from oslo_config import cfg
from oslo_db import exception as db_exc
from oslo_utils import uuidutils
import six
from sqlalchemy import exc as inner_db_exc
from webob import exc

from networking_cisco._i18n import _
from networking_cisco import backwards_compatibility as bc
from networking_cisco.backwards_compatibility import (cb_constants as
    service_constants)
from networking_cisco.backwards_compatibility import (exnet_const as
    external_net)
from networking_cisco.backwards_compatibility import cb_registry as registry
from networking_cisco.backwards_compatibility import dns_const
from networking_cisco.backwards_compatibility import extraroute_const
from networking_cisco.backwards_compatibility import l3_const
import networking_cisco.plugins
from networking_cisco.plugins.cisco.common import cisco_constants as c_const
from networking_cisco.plugins.cisco.db.l3 import l3_router_appliance_db
from networking_cisco.plugins.cisco.device_manager import service_vm_lib
from networking_cisco.plugins.cisco.extensions import ciscohostingdevicemanager
from networking_cisco.plugins.cisco.extensions import routertype
from networking_cisco.tests.unit.cisco.device_manager import (
    device_manager_test_support)
from networking_cisco.tests.unit.cisco.device_manager import (
    test_db_device_manager)
from networking_cisco.tests.unit.cisco.l3 import l3_router_test_support
from networking_cisco.tests.unit.cisco.l3 import test_db_routertype

_uuid = uuidutils.generate_uuid


NEUTRON_VERSION = bc.NEUTRON_VERSION
NEUTRON_NEWTON_VERSION = bc.NEUTRON_NEWTON_VERSION

CORE_PLUGIN_KLASS = device_manager_test_support.CORE_PLUGIN_KLASS
L3_PLUGIN_KLASS = (
    'networking_cisco.tests.unit.cisco.l3.test_l3_router_appliance_plugin.'
    'TestApplianceL3RouterServicePlugin')
extensions_path = networking_cisco.plugins.__path__[0] + '/cisco/extensions'
policy_path = (os.path.abspath(networking_cisco.__path__[0]) +
               '/../etc/policy.json')
VM_ROUTERTYPE_NAME = test_db_device_manager.VM_ROUTERTYPE_NAME


class TestL3RouterApplianceExtensionManager(
        test_db_routertype.L3TestRoutertypeExtensionManager):

    def get_resources(self):
        l3_const.RESOURCE_ATTRIBUTE_MAP['routers'].update(
            extraroute_const.EXTENDED_ATTRIBUTES_2_0['routers'])
        l3_const.RESOURCE_ATTRIBUTE_MAP[l3_const.FLOATINGIPS].update(
            dns_const.EXTENDED_ATTRIBUTES_2_0[l3_const.FLOATINGIPS])
        return super(TestL3RouterApplianceExtensionManager,
                     self).get_resources()


class TestNoL3NatPlugin(test_l3.TestNoL3NatPlugin,
                        agents_db.AgentDbMixin):

    # There is no need to expose agent REST API
    supported_extension_aliases = ["external-net", "provider"]
    NET_TYPE = 'vlan'

    def __init__(self):
        self.tags = {}
        self.tag = 1
        super(TestNoL3NatPlugin, self).__init__()

    def _make_network_dict(self, network, fields=None,
                           process_extensions=True):
        res = {'id': network['id'],
               'name': network['name'],
               'tenant_id': network['tenant_id'],
               'admin_state_up': network['admin_state_up'],
               'status': network['status'],
               'shared': network['shared'],
               'subnets': [subnet['id']
                           for subnet in network['subnets']]}
        try:
            tag = self.tags[network['id']]
        except KeyError:
            self.tag += 1
            tag = self.tag
            self.tags[network['id']] = tag
        res.update({pnet.PHYSICAL_NETWORK: 'phy',
                    pnet.NETWORK_TYPE: self.NET_TYPE,
                    pnet.SEGMENTATION_ID: tag})
        # Call auxiliary extend functions, if any
        if process_extensions:
            self._apply_dict_extend_functions(
                attributes.NETWORKS, res, network)
        return self._fields(res, fields)

    def get_network_profiles(self, context, filters=None, fields=None):
        return [{'id': "1234"}]

    def get_policy_profiles(self, context, filters=None, fields=None):
        return [{'id': "4321"}]


# A set routes capable L3 routing service plugin class supporting appliances
class TestApplianceL3RouterServicePlugin(
        dns_db.DNSDbMixin, l3_router_test_support.TestL3RouterServicePlugin):

    supported_extension_aliases = (
        l3_router_test_support.TestL3RouterServicePlugin.
        supported_extension_aliases + ["extraroute"] + ["dns-integration"])

    def cleanup_after_test(self):
        """Reset all class variables to their default values.
        This is needed to avoid tests to pollute subsequent tests.
        """
        TestApplianceL3RouterServicePlugin._router_schedulers = {}
        TestApplianceL3RouterServicePlugin._router_drivers = {}
        TestApplianceL3RouterServicePlugin._namespace_router_type_id = None
        TestApplianceL3RouterServicePlugin._backlogged_routers = set()
        TestApplianceL3RouterServicePlugin._refresh_router_backlog = True


class L3RouterApplianceTestCaseBase(
    test_db_base_plugin_v2.NeutronDbPluginV2TestCase,
    test_db_routertype.RoutertypeTestCaseMixin,
    test_db_device_manager.DeviceManagerTestCaseMixin,
    l3_router_test_support.L3RouterTestSupportMixin,
        device_manager_test_support.DeviceManagerTestSupportMixin):

    resource_prefix_map = (test_db_device_manager.TestDeviceManagerDBPlugin
                           .resource_prefix_map)
    router_type = None
    configure_routertypes = True
    mock_cfg_agent_notifiers = True

    def setUp(self, core_plugin=None, l3_plugin=None, dm_plugin=None,
              ext_mgr=None, create_mgmt_nw=True, service_plugins=None):
        # Save the global RESOURCE_ATTRIBUTE_MAP
        self.saved_attr_map = {}
        for resource, attrs in six.iteritems(
                attributes.RESOURCE_ATTRIBUTE_MAP):
            self.saved_attr_map[resource] = attrs.copy()
        if not core_plugin:
            core_plugin = CORE_PLUGIN_KLASS
        if l3_plugin is None:
            l3_plugin = L3_PLUGIN_KLASS
        if not service_plugins:
            service_plugins = {'l3_plugin_name': l3_plugin}
        cfg.CONF.set_override('api_extensions_path',
                              l3_router_test_support.extensions_path)

        # for these tests we need to enable overlapping ips
        cfg.CONF.set_default('allow_overlapping_ips', True)
        cfg.CONF.set_default('max_routes', 3)
        if ext_mgr is None:
            ext_mgr = TestL3RouterApplianceExtensionManager()

        super(L3RouterApplianceTestCaseBase, self).setUp(
            plugin=core_plugin, service_plugins=service_plugins,
            ext_mgr=ext_mgr)

        # Ensure we use policy definitions from our repo
        cfg.CONF.set_override('policy_file', policy_path, 'oslo_policy')

        self.core_plugin = bc.get_plugin()
        self.l3_plugin = bc.get_plugin(bc.constants.L3)

        self.setup_notification_driver()

        if NEUTRON_VERSION.version[0] <= NEUTRON_NEWTON_VERSION.version[0]:
            cfg.CONF.set_override('allow_sorting', True)
        self._define_keystone_authtoken()

        cfg.CONF.register_opt(
            cfg.BoolOpt('router_auto_schedule', default=True,
                        help=_('Allow auto scheduling of routers to '
                               'L3 agent.')))
        if self.router_type is not None:
            cfg.CONF.set_override('default_router_type', self.router_type,
                                  group='routing')

        self._mock_l3_admin_tenant()
        self._created_mgmt_nw = create_mgmt_nw
        if create_mgmt_nw is True:
            self._create_mgmt_nw_for_tests(self.fmt)
        if self.configure_routertypes is True:
            templates = self._test_create_hosting_device_templates()
            self._test_create_routertypes(templates.values())
        # in unit tests we don't use keystone so we mock that session
        self.core_plugin._svc_vm_mgr_obj = service_vm_lib.ServiceVMManager(
            True, None, None, None, '', keystone_session=mock.MagicMock())
        self._mock_svc_vm_create_delete(self.core_plugin)
        if self.mock_cfg_agent_notifiers is True:
            self._mock_cfg_agent_notifier(self.l3_plugin)
        # mock the periodic router backlog processing in the tests
        self._mock_backlog_processing(self.l3_plugin)

    def restore_attribute_map(self):
        # Restore the original RESOURCE_ATTRIBUTE_MAP
        attributes.RESOURCE_ATTRIBUTE_MAP = self.saved_attr_map

    def tearDown(self):
        if self.configure_routertypes is True:
            self._test_remove_routertypes()
            self._test_remove_hosting_device_templates()
        if self._created_mgmt_nw is True:
            self._remove_mgmt_nw_for_tests()
        self.l3_plugin.cleanup_after_test()
        self.core_plugin.cleanup_after_test()
        self.restore_attribute_map()
        super(L3RouterApplianceTestCaseBase, self).tearDown()


class L3RouterApplianceRouterTypeDriverTestCase(test_l3.L3NatTestCaseMixin,
                                                L3RouterApplianceTestCaseBase):
    #TODO(bobmel): Add unit tests for the other driver methods when those are
    # actually called

    #NOTE(bobmel): Work-around to make these unit tests to work since we
    # let the core plugin implement the device manager service.
    # The device manager service should map to hosting device extension
    service_constants.EXT_TO_SERVICE_MAPPING[
        ciscohostingdevicemanager.HOSTING_DEVICE_MANAGER_ALIAS] = (
        c_const.DEVICE_MANAGER)
    routertype = test_db_routertype.HW_ROUTERTYPE_NAME

    def setUp(self, core_plugin=None, l3_plugin=None, dm_plugin=None,
              ext_mgr=None):
        super(L3RouterApplianceRouterTypeDriverTestCase, self).setUp(
            core_plugin, l3_plugin, dm_plugin, ext_mgr)
        # Remove any dict extend functions that our plugin does not support
        _dict_extend_functions = l3_router_appliance_db.DICT_EXTEND_FUNCTIONS
        _dict_extend_functions.append('_extend_router_dict_extraroute')
        if bc.NEUTRON_VERSION < bc.NEUTRON_PIKE_VERSION:
            for func in self.plugin._dict_extend_functions[
                    l3_const.ROUTERS][:]:
                if func in l3_router_appliance_db.DICT_EXTEND_FUNCTIONS:
                    continue
                self.plugin._dict_extend_functions[
                        l3_const.ROUTERS].remove(func)

    def test_schedule_router_pre_and_post_commit(self):
        hdts = self._list(
            'hosting_device_templates',
            query_params='name=%s' % test_db_device_manager.HW_TEMPLATE_NAME)
        hdt_id = hdts['hosting_device_templates'][0]['id']
        with mock.patch.object(
                self.l3_plugin, '_refresh_router_backlog', False),\
                mock.patch('networking_cisco.plugins.cisco.l3.drivers.'
                           'noop_routertype_driver.NoopL3RouterDriver.'
                           'schedule_router_precommit') as pre_mock,\
                mock.patch('networking_cisco.plugins.cisco.l3.drivers.'
                       'noop_routertype_driver.NoopL3RouterDriver.'
                       'schedule_router_postcommit') as post_mock,\
                mock.patch('networking_cisco.plugins.cisco.db.l3.'
                           'l3_router_appliance_db.L3RouterApplianceDBMixin.'
                           '_get_router_type_scheduler') as scheduler_mock,\
                mock.patch.object(
                    self.core_plugin,
                    'acquire_hosting_device_slots') as acquire_mock,\
                self.hosting_device(hdt_id) as hosting_device:
            hd = hosting_device['hosting_device']
            scheduler_mock.return_value.schedule_router.return_value = (
                [hd['id']])
            acquire_mock.return_value = True
            with self.router():
                self.l3_plugin._process_backlogged_routers()
                pre_mock.assert_has_calls([mock.call(mock.ANY, mock.ANY)])
                post_mock.assert_has_calls([mock.call(mock.ANY, mock.ANY)])

    def test_unschedule_router_pre_and_post_commit(self):
        with mock.patch('networking_cisco.plugins.cisco.l3.drivers.'
                        'noop_routertype_driver.NoopL3RouterDriver.'
                        'unschedule_router_precommit') as pre_mock,\
                mock.patch('networking_cisco.plugins.cisco.l3.drivers.'
                           'noop_routertype_driver.NoopL3RouterDriver.'
                           'unschedule_router_postcommit') as post_mock,\
                mock.patch('networking_cisco.plugins.cisco.db.l3.'
                           'l3_router_appliance_db.L3RouterApplianceDBMixin.'
                           '_get_router_type_scheduler') as scheduler_mock,\
                mock.patch.object(self.l3_plugin, '_get_effective_slot_need',
                                  return_value=5),\
                mock.patch.object(self.core_plugin,
                                  'release_hosting_device_slots') as (
                    release_mock):
            scheduler_mock.return_value.unschedule_router.return_value = True
            release_mock.return_value = True
            with self.router() as router:
                r = router['router']
                binding_mock = mock.MagicMock()
                binding_mock.router_id = r['id']
                binding_mock.router_type_id = r[routertype.TYPE_ATTR]
                r['gw_port_id'] = None
                r['route_list'] = []
                binding_mock.router.__getitem__.side_effect = r.__getitem__
                binding_mock.router.hosting_info = None
                self.l3_plugin.unschedule_router_from_hosting_device(
                    bc.context.get_admin_context(), binding_mock)
            pre_mock.assert_has_calls([mock.call(mock.ANY, mock.ANY)])
            post_mock.assert_has_calls([mock.call(mock.ANY, mock.ANY)])

    def test_create_router_pre_and_post(self):
        driver = mock.Mock()
        self.l3_plugin._get_router_type_driver = mock.Mock(
            return_value=driver)
        router = {'router': {'tenant_id': 'foo',
                             'admin_state_up': True,
                             'name': 'bar'}}
        ctx = bc.context.get_admin_context()
        self.l3_plugin.create_router(ctx, router)
        driver.create_router_precommit.assert_called_once_with(
            ctx, mock.ANY)
        driver.create_router_postcommit.assert_called_once_with(
            ctx, mock.ANY)

    def test_update_router_pre_and_post(self):
        driver = mock.Mock()
        self.l3_plugin._get_router_type_driver = mock.Mock(
            return_value=driver)
        with self.router() as router:
            r = router['router']
            ctx = bc.context.get_admin_context()
            self.l3_plugin.update_router(ctx, r['id'], router)
            driver.update_router_precommit.assert_called_once_with(
                ctx, mock.ANY)
            driver.update_router_postcommit.assert_called_once_with(
                ctx, mock.ANY)

    def test_delete_router_pre_and_post(self):
        driver = mock.Mock()
        self.l3_plugin._get_router_type_driver = mock.Mock(
            return_value=driver)
        with self.router() as router:
            r = router['router']
            ctx = bc.context.get_admin_context()
            self.l3_plugin.delete_router(ctx, r['id'])
            driver.delete_router_precommit.assert_called_once_with(
                ctx, mock.ANY)
            driver.delete_router_postcommit.assert_called_once_with(
                ctx, mock.ANY)

    def test_add_router_interface_pre_and_post_subnet(self):
        driver = mock.Mock()
        self.l3_plugin._get_router_type_driver = mock.Mock(
            return_value=driver)
        with self.router() as router, self.subnet(cidr='10.0.1.0/24') as sub:
            r = router['router']
            s1 = sub['subnet']
            ctx = bc.context.get_admin_context()
            info = {'subnet_id': s1['id']}
            self.l3_plugin.add_router_interface(ctx, r['id'], info)
            driver.add_router_interface_precommit.assert_called_once_with(
                ctx, mock.ANY)
            driver.add_router_interface_postcommit.assert_called_once_with(
                ctx, mock.ANY)

    def test_add_router_interface_pre_and_post_port(self):
        driver = mock.Mock()
        self.l3_plugin._get_router_type_driver = mock.Mock(
            return_value=driver)
        with self.router() as router, self.port(cidr='10.0.1.0/24') as port:
            r = router['router']
            p1 = port['port']
            ctx = bc.context.get_admin_context()
            info = {'port_id': p1['id']}
            self.l3_plugin.add_router_interface(ctx, r['id'], info)
            driver.add_router_interface_precommit.assert_called_once_with(
                ctx, mock.ANY)
            driver.add_router_interface_postcommit.assert_called_once_with(
                ctx, mock.ANY)

    def test_add_router_interface_dns(self):
        driver = mock.Mock()
        self.l3_plugin._get_router_type_driver = mock.Mock(
            return_value=driver)
        with self.router() as router, self.port(cidr='10.0.1.0/24') as port:
            r = router['router']
            port['dns_name'] = "test_dns_name"
            port['dns_domain'] = "test_dns_domain"
            p1 = port['port']
            ctx = bc.context.get_admin_context()
            info = {'port_id': p1['id']}
            self.l3_plugin.add_router_interface(ctx, r['id'], info)
            driver.add_router_interface_precommit.assert_called_once_with(
                ctx, mock.ANY)
            driver.add_router_interface_postcommit.assert_called_once_with(
                ctx, mock.ANY)

    def test_remove_router_interface_pre_and_post_subnet(self):
        driver = mock.Mock()
        self.l3_plugin._get_router_type_driver = mock.Mock(
            return_value=driver)
        with self.router() as router, self.subnet(cidr='10.0.1.0/24') as sub:
            r = router['router']
            s1 = sub['subnet']
            ctx = bc.context.get_admin_context()
            info = {'subnet_id': s1['id']}
            self.l3_plugin.add_router_interface(ctx, r['id'], info)
            self.l3_plugin.remove_router_interface(ctx, r['id'], info)
            driver.remove_router_interface_precommit.assert_called_once_with(
                ctx, mock.ANY)
            driver.remove_router_interface_postcommit.assert_called_once_with(
                ctx, mock.ANY)

    def test_remove_router_interface_pre_and_post_port(self):
        driver = mock.Mock()
        self.l3_plugin._get_router_type_driver = mock.Mock(
            return_value=driver)
        with self.router() as router, self.port(cidr='10.0.1.0/24') as port:
            r = router['router']
            p1 = port['port']
            ctx = bc.context.get_admin_context()
            info = {'port_id': p1['id']}
            self.l3_plugin.add_router_interface(ctx, r['id'], info)
            self.l3_plugin.remove_router_interface(ctx, r['id'], info)
            driver.remove_router_interface_precommit.assert_called_once_with(
                ctx, mock.ANY)
            driver.remove_router_interface_postcommit.assert_called_once_with(
                ctx, mock.ANY)

    def test_create_floating_ip_post(self):
        driver = mock.Mock()
        self.l3_plugin._get_router_type_driver = mock.Mock(
            return_value=driver)
        with self.subnet() as ext_s, self.subnet(cidr='10.0.1.0/24') as s:
            s1 = ext_s['subnet']
            ext_net_id = s1['network_id']
            self._set_net_external(ext_net_id)
            with self.router(
                    external_gateway_info={'network_id': ext_net_id}) as r,\
                    self.port(s) as p:
                self._router_interface_action('add', r['router']['id'], None,
                                              p['port']['id'])
                p1 = p['port']
                fip = {'floatingip': {'floating_network_id': ext_net_id,
                                      'port_id': p1['id'],
                                      'tenant_id': s1['tenant_id']}}
                ctx = bc.context.get_admin_context()
                self.l3_plugin.create_floatingip(ctx, fip)
                driver.create_floatingip_postcommit.assert_called_once_with(
                    ctx, mock.ANY)

    def test_create_floating_ip_post_dns(self):
        driver = mock.Mock()
        self.l3_plugin._get_router_type_driver = mock.Mock(
            return_value=driver)
        with self.subnet() as ext_s, self.subnet(cidr='10.0.1.0/24') as s:
            s1 = ext_s['subnet']
            ext_net_id = s1['network_id']
            self._set_net_external(ext_net_id)
            with self.router(
                    external_gateway_info={'network_id': ext_net_id}) as r,\
                    self.port(s) as p:
                self._router_interface_action('add', r['router']['id'], None,
                                              p['port']['id'])
                p1 = p['port']
                fip = {'floatingip': {'floating_network_id': ext_net_id,
                                      'port_id': p1['id'],
                                      'tenant_id': s1['tenant_id'],
                                      'dns_name': 'dns_test',
                                      'dns_domain': 'dns_domain'}}
                ctx = bc.context.get_admin_context()
                self.l3_plugin.create_floatingip(ctx, fip)
                driver.create_floatingip_postcommit.assert_called_once_with(
                    ctx, mock.ANY)
                #JAJ - test for dns_name on FIP?  How?

    def test_update_floating_ip_pre_and_post(self):
        driver = mock.Mock()
        self.l3_plugin._get_router_type_driver = mock.Mock(
            return_value=driver)
        with self.subnet() as ext_s, self.subnet(cidr='10.0.1.0/24') as s:
            s1 = ext_s['subnet']
            ext_net_id = s1['network_id']
            self._set_net_external(ext_net_id)
            with self.router(
                    external_gateway_info={'network_id': ext_net_id}) as r,\
                    self.port(s) as p:
                self._router_interface_action('add', r['router']['id'], None,
                                              p['port']['id'])
                p1 = p['port']
                fip = {'floatingip': {'floating_network_id': ext_net_id,
                                      'port_id': p1['id'],
                                      'tenant_id': s1['tenant_id']}}
                ctx = bc.context.get_admin_context()
                floating_ip = self.l3_plugin.create_floatingip(ctx, fip)
                fip = {'floatingip': {'port_id': None}}
                self.l3_plugin.update_floatingip(ctx, floating_ip['id'], fip)
                driver.update_floatingip_precommit.assert_called_once_with(
                    ctx, mock.ANY)
                fip = {'floatingip': {'floating_network_id': ext_net_id,
                                      'port_id': p1['id'],
                                      'tenant_id': s1['tenant_id']}}
                self.l3_plugin.update_floatingip(ctx, floating_ip['id'], fip)
                driver.update_floatingip_postcommit.assert_called_once_with(
                    ctx, mock.ANY)

    def test_delete_floating_ip_pre_and_post(self):
        driver = mock.Mock()
        self.l3_plugin._get_router_type_driver = mock.Mock(
            return_value=driver)
        with self.subnet() as ext_s, self.subnet(cidr='10.0.1.0/24') as s:
            s1 = ext_s['subnet']
            ext_net_id = s1['network_id']
            self._set_net_external(ext_net_id)
            with self.router(
                    external_gateway_info={'network_id': ext_net_id}) as r,\
                    self.port(s) as p:
                self._router_interface_action('add', r['router']['id'], None,
                                              p['port']['id'])
                p1 = p['port']
                fip = {'floatingip': {'floating_network_id': ext_net_id,
                                      'port_id': p1['id'],
                                      'tenant_id': s1['tenant_id']}}
                ctx = bc.context.get_admin_context()
                floating_ip = self.l3_plugin.create_floatingip(ctx, fip)
                self.l3_plugin.delete_floatingip(ctx, floating_ip['id'])
                driver.delete_floatingip_precommit.assert_called_once_with(
                    ctx, mock.ANY)
                driver.delete_floatingip_postcommit.assert_called_once_with(
                    ctx, mock.ANY)


class L3RouterApplianceNamespaceTestCase(
    test_l3.L3NatTestCaseBase, test_extraroute.ExtraRouteDBTestCaseBase,
        L3RouterApplianceTestCaseBase):

    router_type = c_const.NAMESPACE_ROUTER_TYPE

    def test_floatingip_with_assoc_fails(self):
        self._test_floatingip_with_assoc_fails(
            'neutron.db.l3_db.L3_NAT_dbonly_mixin._check_and_get_fip_assoc')

    def _check_driver_calls(self, func_name, num_teardown, num_setup):
        with mock.patch.object(self.core_plugin,
                               'get_hosting_device_plugging_driver') as m:
            func = getattr(super(L3RouterApplianceNamespaceTestCase, self),
                           func_name, None)
            # call test case function
            func()
            drv = m.return_value
            teardown_mock = drv.teardown_logical_port_connectivity
            setup_mock = drv.setup_logical_port_connectivity
            self.assertEqual(teardown_mock.call_count, num_teardown)
            self.assertEqual(setup_mock.call_count, num_setup)

    def test_router_update_gateway_with_external_ip_used_by_gw(self):
        self._check_driver_calls(
            'test_router_update_gateway_with_external_ip_used_by_gw', 0, 0)

    def test_router_update_gateway_with_invalid_external_ip(self):
        self._check_driver_calls(
            'test_router_update_gateway_with_invalid_external_ip', 0, 0)

    def test_router_update_gateway_with_invalid_external_subnet(self):
        self._check_driver_calls(
            'test_router_update_gateway_with_invalid_external_subnet', 0, 0)

    def test_router_update_gateway_with_existed_floatingip(self):
        self._check_driver_calls(
            'test_router_update_gateway_with_existed_floatingip', 1, 1)

    def test_router_update_gateway_to_empty_with_existed_floatingip(self):
        self._check_driver_calls(
            'test_router_update_gateway_to_empty_with_existed_floatingip', 1,
            1)

    def _test_rest_api_operations_denied_for_plugin_managed_router(
            self, func, *args, **kwargs):
            with mock.patch.object(self.l3_plugin, '_get_router_binding_info',
                                   return_value=lambda: None) as m:
                # for now operations on routers of any role != None are not
                # permitted for anyone but the l3plugin itself
                setattr(m.return_value, 'role', 'some_role')
                func(*args, **kwargs)

    def test_router_update_denied_for_plugin_managed_router(self):
        with self.router() as router:
            self._test_rest_api_operations_denied_for_plugin_managed_router(
                self._update, 'routers', router['router']['id'],
                {'router': {'name': 'new_name'}},
                expected_code=exc.HTTPForbidden.code)

    def test_router_delete_denied_for_plugin_managed_router(self):
        with self.router() as router:
            self._test_rest_api_operations_denied_for_plugin_managed_router(
                self._delete, 'routers', router['router']['id'],
                expected_code=exc.HTTPForbidden.code)

    def test_router_interface_add_denied_for_plugin_managed_router(self):
        with self.router() as router:
            self._test_rest_api_operations_denied_for_plugin_managed_router(
                self._router_interface_action, 'add', router['router']['id'],
                'some_subnet_id', None, expected_code=exc.HTTPForbidden.code)

    def router_interface_remove_denied_for_plugin_managed_router(self):
        with self.router() as router:
            self._test_rest_api_operations_denied_for_plugin_managed_router(
                self._router_interface_action, 'remove',
                router['router']['id'], 'some_subnet_id', None,
                expected_code=exc.HTTPForbidden.code)


class L3RouterApplianceVMTestCase(L3RouterApplianceNamespaceTestCase):

    router_type = VM_ROUTERTYPE_NAME

    def setUp(self, core_plugin=None, l3_plugin=None, dm_plugin=None,
              ext_mgr=None):
        super(L3RouterApplianceVMTestCase, self).setUp(
            core_plugin=core_plugin, l3_plugin=l3_plugin, dm_plugin=dm_plugin,
            ext_mgr=ext_mgr)

        self._mock_get_routertype_scheduler_always_none()

    @unittest.skipIf(bc.NEUTRON_VERSION < bc.NEUTRON_NEWTON_VERSION,
                     "Test not applicable prior to Newton")
    def test_create_router_gateway_fails_nested_delete_router_failed(self):
        (super(L3RouterApplianceNamespaceTestCase, self).
         test_create_router_gateway_fails_nested_delete_router_failed())
        # must disable the UT patch for 'delete_router' so that our router
        # type cleanup, which deletes any remaining routers, can proceed
        if bc.NEUTRON_VERSION < bc.NEUTRON_PIKE_VERSION:
            # be lazy and stop all
            mock.patch.stopall()
        else:
            # In pike, during general cleanup, there is an explicit call to
            # stop one of the other patches, so cannot do stop all here as
            # that trigger an exception during that general cleanup.
            # Need a patch here for the sole purpose of determining the
            # existing patches, so patching this very function.
            dummy_patch = mock.patch(
                'L3RouterApplianceVMTestCase.'
                'test_create_router_gateway_fails_nested_delete_router_failed')
            delete_router_patch = None
            for p in dummy_patch._active_patches:
                if p.attribute == 'delete_router':
                    delete_router_patch = p
                    break
            if delete_router_patch:
                delete_router_patch.stop()


class L3AgentRouterApplianceTestCase(L3RouterApplianceTestCaseBase,
                                     test_l3.L3AgentDbTestCaseBase):

    router_type = c_const.NAMESPACE_ROUTER_TYPE

    def setUp(self, core_plugin=None, l3_plugin=None, dm_plugin=None,
              ext_mgr=None):
        super(L3AgentRouterApplianceTestCase, self).setUp(
            core_plugin=core_plugin, l3_plugin=l3_plugin, dm_plugin=dm_plugin,
            ext_mgr=ext_mgr)
        # UTs in parent class expects self.plugin to refer to l3 plugin
        self.plugin = self.l3_plugin

    # Overloaded test function that needs to be modified to run
    @unittest.skipIf(bc.NEUTRON_VERSION < bc.NEUTRON_NEWTON_VERSION,
                     "Test not applicable prior to Newton")
    def test_router_delete_event_exception_preserved(self):
        super(L3AgentRouterApplianceTestCase,
              self).test_router_delete_event_exception_preserved()
        registry.clear()

    def _test_notify_op_agent(self, target_func, *args):
        kargs = [item for item in args]
        kargs.append(self._l3_agent_mock)
        target_func(*kargs)


class L3CfgAgentRouterApplianceTestCase(L3RouterApplianceTestCaseBase,
                                        test_l3.L3AgentDbTestCaseBase):

    def setUp(self, core_plugin=None, l3_plugin=None, dm_plugin=None,
              ext_mgr=None):
        super(L3CfgAgentRouterApplianceTestCase, self).setUp(
            core_plugin=core_plugin, l3_plugin=l3_plugin, dm_plugin=dm_plugin,
            ext_mgr=ext_mgr)
        self.orig_get_sync_data = self.l3_plugin.get_sync_data
        self.l3_plugin.get_sync_data = self.l3_plugin.get_sync_data_ext
        self._mock_get_routertype_scheduler_always_none()
        # Some UTs in parent class expects self.plugin to refer to l3 plugin
        self.plugin = self.l3_plugin

    def tearDown(self):
        self.l3_plugin.get_sync_data = self.orig_get_sync_data
        super(L3CfgAgentRouterApplianceTestCase, self).tearDown()

    # Overloaded test function that needs to be modified to run
    @unittest.skipIf(bc.NEUTRON_VERSION < bc.NEUTRON_NEWTON_VERSION,
                     "Test not applicable prior to Newton")
    def test_router_delete_event_exception_preserved(self):
        super(L3CfgAgentRouterApplianceTestCase,
              self).test_router_delete_event_exception_preserved()
        registry.clear()

    def _test_notify_op_agent(self, target_func, *args):
        kargs = [item for item in args]
        kargs.append(self._l3_cfg_agent_mock)
        target_func(*kargs)

    def test_failed_add_gw_hosting_port_info_changes_router_status(self,
                                                                   num=1):
        with self.subnet() as ext_s, self.subnet(cidr='10.0.1.0/24') as s:
            ext_net_id = ext_s['subnet']['network_id']
            self._set_net_external(ext_net_id)
            with self.router(
                    external_gateway_info={'network_id': ext_net_id}) as r,\
                    self.port(s) as p:
                self._router_interface_action('add', r['router']['id'], None,
                                              p['port']['id'])
                with mock.patch.object(
                        self.l3_plugin,
                        'add_type_and_hosting_device_info') as m1,\
                        mock.patch.object(
                            self.l3_plugin,
                            '_populate_hosting_info_for_port') as m2,\
                        mock.patch.object(
                            self._core_plugin,
                            'get_hosting_device_plugging_driver'):
                    m1.side_effect = lambda ctx, r, bi=None, sch=None: (
                        r.update({'hosting_device': {'id': 'fake_id'}}))
                    m2.return_value = None
                    routers = self.l3_plugin.get_sync_data(
                        bc.context.get_admin_context(), None)
                    self.assertEqual(num, len(routers))
                    self.assertEqual(c_const.ROUTER_INFO_INCOMPLETE,
                                     routers[0]['status'])
                    self.assertEqual(num, m2.call_count)

    def test_failed_add_hosting_port_info_changes_router_status(self, num=1):
        with self.router() as r, self.subnet(cidr='10.0.1.0/24') as s2:
            with self.port() as p1, self.port(s2) as p2:
                r_id = r['router']['id']
                self._router_interface_action('add', r_id, None,
                                              p1['port']['id'])
                self._router_interface_action('add', r_id, None,
                                              p2['port']['id'])
                with mock.patch.object(
                        self.l3_plugin,
                        'add_type_and_hosting_device_info') as m1,\
                        mock.patch.object(
                            self.l3_plugin,
                            '_populate_hosting_info_for_port') as m2,\
                        mock.patch.object(
                            self._core_plugin,
                            'get_hosting_device_plugging_driver'):
                    m1.side_effect = lambda ctx, r, bi=None, sch=None: (
                        r.update({'hosting_device': {'id': 'fake_id'}}))
                    m2.return_value = None
                    routers = self.l3_plugin.get_sync_data(
                        bc.context.get_admin_context(), None)
                    self.assertEqual(num, len(routers))
                    self.assertEqual(c_const.ROUTER_INFO_INCOMPLETE,
                                     routers[0]['status'])
                    self.assertEqual(num, m2.call_count)

    def test__allocate_hosting_port_returns_none_if_binding_fails(self):
        with self.router() as r, self.port() as p1:
            r_id = r['router']['id']
            p1_id = p1['port']['id']
            self._router_interface_action('add', r_id, None, p1_id)
            plugging_drv_mock = mock.MagicMock()
            plugging_drv_mock.allocate_hosting_port.return_value = {
                'allocated_port_id': p1_id, 'allocated_vlan': 10}
            adm_ctx = bc.context.get_admin_context()
            p1_db = self._core_plugin._get_port(adm_ctx, p1_id)
            ctx_mock = mock.MagicMock()
            ctx_mock.session = mock.MagicMock()
            ctx_mock.session.begin = adm_ctx.session.begin
            ctx_mock.session.add = mock.MagicMock()
            ctx_mock.session.add.side_effect = db_exc.DBReferenceError(
                'cisco_port_mappings', 'foreign key constraint', p1_id,
                'ports', inner_exception=inner_db_exc.IntegrityError(
                    "Invalid insert", params="", orig=None))
            res = self.l3_plugin._allocate_hosting_port(
                ctx_mock, r_id, p1_db, 'fake_hd_id', plugging_drv_mock)
            self.assertIsNone(res)


class L3RouterApplianceGbpTestCase(test_l3.L3NatTestCaseMixin,
                                   L3RouterApplianceTestCaseBase):

    router_type = "ASR1k_Neutron_router"

    def setUp(self, core_plugin=None, l3_plugin=None, dm_plugin=None,
              ext_mgr=None):
        super(L3RouterApplianceGbpTestCase, self).setUp(
            core_plugin=core_plugin, l3_plugin=l3_plugin, dm_plugin=dm_plugin,
            ext_mgr=ext_mgr)
        self._created_mgmt_nw = False

    @contextlib.contextmanager
    def _mock_neutron_service_plugins(self):
        """Context manager for mocking get_plugin

        This is required to mock and unmock the function as close to where its
        being used as possible.
        """
        with mock.patch.object(bc, 'get_plugin') as get_plugin:
            get_plugin.side_effect = lambda svc='CORE': {
                'CORE': self.core_plugin, 'GROUP_POLICY': object(),
                bc.constants.L3: self}[svc]
            yield get_plugin

    def test_is_gbp_workflow(self):
        with self._mock_neutron_service_plugins():
            self.assertTrue(self.l3_plugin.is_gbp_workflow)

    def test_create_floatingip_gbp(self):
        kwargs = {'arg_list': (external_net.EXTERNAL,),
                  external_net.EXTERNAL: True}
        self.l3_plugin._update_fip_assoc = mock.Mock(return_value={})
        with self.network(**kwargs) as net:
            with self.subnet(network=net, cidr='200.0.0.0/22') as sub:
                subnet = sub['subnet']
                # dummy func is used to verify that our stub was called
                dummy_func = mock.Mock()

                def _stub_modify_context(context, fip_context):
                    context.nat_pool_list = [{'subnet_id': subnet['id']}]
                    dummy_func(context)

                mock_drvr = mock.Mock()
                mock_drvr.create_floatingip_precommit = _stub_modify_context
                self.l3_plugin._get_router_type_driver = mock.Mock(
                    return_value=mock_drvr
                )
                network = net['network']
                floating_ip = {
                    'floatingip': {'floating_network_id': network['id'],
                                   'tenant_id': net['network']['tenant_id']}
                }
                ctx = bc.context.get_admin_context()
                with self._mock_neutron_service_plugins():
                    self.l3_plugin.create_floatingip(ctx, floating_ip)
                dummy_func.assert_called_once_with(ctx)
                mock_drvr.create_floatingip_postcommit.assert_called_once_with(
                    ctx, mock.ANY)
                self.l3_plugin._update_fip_assoc.assert_called_once_with(ctx,
                    mock.ANY, mock.ANY, mock.ANY)

    def test_update_floatingip_gbp(self):
        self.l3_plugin._do_update_floatingip = mock.Mock()
        ctx = bc.context.get_admin_context()
        TEST_FIP_UUID = _uuid()
        floating_ip = {
            'floatingip': {'floating_network_id': _uuid()}
        }
        with self._mock_neutron_service_plugins():
            self.l3_plugin.update_floatingip(ctx, TEST_FIP_UUID, floating_ip)
        self.l3_plugin._do_update_floatingip.assert_called_once_with(ctx,
            TEST_FIP_UUID, floating_ip, add_fip=True)


class L3RouterApplianceNoGbpTestCase(test_l3.L3NatTestCaseMixin,
                                     L3RouterApplianceTestCaseBase):
    router_type = "ASR1k_Neutron_router"

    def setUp(self, core_plugin=None, l3_plugin=None, dm_plugin=None,
              ext_mgr=None):
        super(L3RouterApplianceNoGbpTestCase, self).setUp(
            core_plugin=core_plugin, l3_plugin=l3_plugin, dm_plugin=dm_plugin,
            ext_mgr=ext_mgr)
        self._created_mgmt_nw = False

    def test_is_not_gbp_workflow(self):
        self.assertFalse(self.l3_plugin.is_gbp_workflow)

    def test_create_floatingip_gbp(self):
        self.l3_plugin._update_fip_assoc = mock.Mock()
        self.l3_plugin._create_floatingip_neutron = mock.Mock()
        self.l3_plugin._create_floatingip_gbp = mock.Mock()
        ctx = bc.context.get_admin_context()
        floating_ip = {
            'floatingip': {'floating_network_id': _uuid()}
        }
        self.l3_plugin.create_floatingip(ctx, floating_ip)
        self.l3_plugin._create_floatingip_gbp.assert_not_called()
        self.l3_plugin._create_floatingip_neutron.assert_called_once_with(ctx,
            floating_ip, initial_status=bc.constants.FLOATINGIP_STATUS_ACTIVE)

    def test_update_floatingip_no_gbp(self):
        self.l3_plugin._do_update_floatingip = mock.Mock()
        ctx = bc.context.get_admin_context()
        TEST_FIP_UUID = _uuid()
        floating_ip = {
            'floatingip': {'floating_network_id': _uuid()}
        }
        self.l3_plugin.update_floatingip(ctx, TEST_FIP_UUID, floating_ip)
        self.l3_plugin._do_update_floatingip.assert_called_once_with(ctx,
            TEST_FIP_UUID, floating_ip)
